import unittest
import rpy2.robjects as robjects
rinterface = robjects.rinterface
import array

import sys
if sys.version_info[0] == 2:
    py3str = unicode
    py3bytes = str
else:
    py3str = str
    py3bytes = bytes

class RInstanceTestCase(unittest.TestCase):

    def tearDow(self):
        robjects.r._dotter = False

    def testGetItem(self):
        letters_R = robjects.r["letters"]
        self.assertTrue(isinstance(letters_R, robjects.Vector))
        letters = (('a', 0), ('b', 1), ('c', 2), ('x', 23), ('y', 24), ('z', 25))
        for l, i in letters:
            self.assertTrue(letters_R[i] == l)
        
        as_list_R = robjects.r["as.list"]
        seq_R = robjects.r["seq"]
        
        mySeq = seq_R(0, 10)
        
        myList = as_list_R(mySeq)
        
        for i, li in enumerate(myList):
            self.assertEqual(i, myList[i][0])

    def testEval(self):
        # vector long enough to span across more than one line
        x = robjects.baseenv['seq'](1, 50, 2)
        res = robjects.r('sum(%s)' %x.r_repr())
        self.assertEqual(625, res[0])
        
class MappingTestCase(unittest.TestCase):

    def testMapperR2Python_string(self):
        sexp = rinterface.globalenv.get("letters")
        ob = robjects.default_converter.ri2ro(sexp)
        self.assertTrue(isinstance(ob, 
                                   robjects.Vector))

    def testMapperR2Python_boolean(self):
        sexp = rinterface.globalenv.get("T")
        ob = robjects.default_converter.ri2ro(sexp)
        self.assertTrue(isinstance(ob, 
                                   robjects.Vector))

    def testMapperR2Python_function(self):
        sexp = rinterface.globalenv.get("plot")
        ob = robjects.default_converter.ri2ro(sexp)
        self.assertTrue(isinstance(ob, 
                                   robjects.Function))

    def testMapperR2Python_environment(self):
        sexp = rinterface.globalenv.get(".GlobalEnv")
        self.assertTrue(isinstance(robjects.default_converter.ri2ro(sexp), 
                                   robjects.Environment))

    def testMapperR2Python_s4(self):
        robjects.r('setClass("A", representation(x="integer"))')
        classname = rinterface.StrSexpVector(["A", ])
        one = rinterface.IntSexpVector([1, ])
        sexp = rinterface.globalenv.get("new")(classname, 
                                               x=one)
        self.assertTrue(isinstance(robjects.default_converter.ri2ro(sexp), 
                                   robjects.RS4))

    def testMapperPy2R_integer(self):
        py = 1
        rob = robjects.default_converter.py2ro(py)
        self.assertTrue(isinstance(rob, robjects.Vector))
        self.assertEqual(rinterface.INTSXP, rob.typeof)

    def testMapperPy2R_boolean(self):        
        py = True
        rob = robjects.default_converter.py2ro(py)
        self.assertTrue(isinstance(rob, robjects.Vector))
        self.assertEqual(rinterface.LGLSXP, rob.typeof)

    def testMapperPy2R_py3bytes(self):        
        py = b'houba'
        rob = robjects.default_converter.py2ro(py)
        self.assertTrue(isinstance(rob, robjects.Vector))
        self.assertEqual(rinterface.STRSXP, rob.typeof)

    def testMapperPy2R_py3str(self):        
        py = u'houba'
        self.assertTrue(isinstance(py, py3str))
        rob = robjects.default_converter.py2ro(py)
        self.assertTrue(isinstance(rob, robjects.Vector))
        self.assertEqual(rinterface.STRSXP, rob.typeof)
        #FIXME: more tests

    def testMapperPy2R_float(self):
        py = 1.0
        rob = robjects.default_converter.py2ro(py)
        self.assertTrue(isinstance(rob, robjects.Vector))
        self.assertEqual(rinterface.REALSXP, rob.typeof)

    def testMapperPy2R_complex(self):
        py = 1.0 + 2j
        rob = robjects.default_converter.py2ro(py)
        self.assertTrue(isinstance(rob, robjects.Vector))
        self.assertEqual(rinterface.CPLXSXP, rob.typeof)

    def testMapperPy2R_taggedlist(self):
        py = robjects.rlc.TaggedList(('a', 'b'),
                                     tags=('foo', 'bar'))
        robj = robjects.default_converter.py2ro(py)
        self.assertTrue(isinstance(robj, robjects.Vector))
        self.assertEqual(2, len(robj))
        self.assertEqual(('foo', 'bar'),
                         tuple(robj.names))

    def testMapperPy2R_function(self):
        func = lambda x: x
        rob = robjects.default_converter.py2ro(func)
        self.assertTrue(isinstance(rob, robjects.SignatureTranslatedFunction))
        self.assertEqual(rinterface.CLOSXP, rob.typeof)


    def testOverride_ri2ro(self):
        class Density(object):
            def __init__(self, x):
                self._x = x

        def f(obj):
            pyobj = robjects.default_converter.ri2ro(obj)
            inherits = rinterface.baseenv["inherits"]
            classname = rinterface.SexpVector(["density", ], 
                                              rinterface.STRSXP)
            if inherits(pyobj, classname)[0]:
                pyobj = Density(pyobj)
            return pyobj
        robjects.conversion.ri2ro = f
        x = robjects.r.rnorm(100)
        d = robjects.r.density(x)

        self.assertTrue(isinstance(d, Density))

class RSlotsTestCase(unittest.TestCase):

    def testItems(self):
        v = robjects.IntVector((1,2,3))
        rs = robjects.robject.RSlots(v)
        self.assertEqual(0, len(tuple(rs.items())))

        v.do_slot_assign("a", robjects.IntVector((9,)))
        for ((k_o,v_o), (k,v)) in zip((("a", robjects.IntVector), ), rs.items()):
            self.assertEqual(k_o, k)
            self.assertEqual(v_o, type(v))
            
    
def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(RInstanceTestCase)
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(MappingTestCase))
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(RSlotsTestCase))
    return suite

if __name__ == '__main__':
     unittest.main()



import rpy2.rinterface as rinterface
from rpy2.robjects.robject import RObjectMixin, RObject
from rpy2.robjects import conversion

_new_env = rinterface.baseenv["new.env"]

class Environment(RObjectMixin, rinterface.SexpEnvironment):
    """ An R environement. """
    
    def __init__(self, o=None):
        if o is None:
            o = _new_env(hash=rinterface.SexpVector([True, ], 
                                                    rinterface.LGLSXP))
        super(Environment, self).__init__(o)

    def __getitem__(self, item):
        res = super(Environment, self).__getitem__(item)
        res = conversion.converter.ri2ro(res)
        # objects in a R environment have an associated name / symbol
        try:
            res.__rname__ = item
        except AttributeError:
            # the 3rd-party conversion function can return objects
            # for which __rname__ cannot be set (because of fixed
            # __slots__ and no __rname__ in the original set
            # of attributes)
            pass
        return res

    def __setitem__(self, item, value):
        robj = conversion.converter.py2ri(value)
        super(Environment, self).__setitem__(item, robj)

    def get(self, item, wantfun = False):
        """ Get a object from its R name/symol
        :param item: string (name/symbol)
        :rtype: object (as returned by :func:`conversion.converter.ri2ro`)
        """
        res = super(Environment, self).get(item, wantfun = wantfun)
        res = conversion.converter.ri2ro(res)
        res.__rname__ = item
        return res

    def keys(self):
        """ Return a tuple listing the keys in the object """
        return tuple([x for x in self])

    def items(self):
        """ Iterate through the symbols and associated objects in
            this R environment."""
        for k in self:
            yield (k, self[k])



import os, re
from collections import OrderedDict
from rpy2.robjects.robject import RObjectMixin, RObject
import rpy2.rinterface as rinterface
from rpy2.robjects import help
#import rpy2.robjects.conversion conversion
from . import conversion
# XXX: I need to import default_ri2ro

from rpy2.robjects.packages_utils import (default_symbol_r2python,
                                          default_symbol_check_after,
                                          _map_symbols,
                                          _fix_map_symbols)


baseenv_ri = rinterface.baseenv

#needed to avoid circular imports
_reval = rinterface.baseenv['eval']

__formals = baseenv_ri.get('formals')
__args = baseenv_ri.get('args')
#_genericsargsenv = baseenv_ri['.GenericArgsEnv']
#_argsenv = baseenv_ri['.ArgsEnv']
def _formals_fixed(func):
    tmp = __args(func)
    return __formals(tmp)

## docstring_property and DocstringProperty
## from Bradley Froehle
## https://gist.github.com/bfroehle/4041015
def docstring_property(class_doc):
    def wrapper(fget):
        return DocstringProperty(class_doc, fget)
    return wrapper
 
class DocstringProperty(object):
    def __init__(self, class_doc, fget):
        self.fget = fget
        self.class_doc = class_doc
    def __get__(self, obj, objtype=None):
        if obj is None:
            return self.class_doc
        else:
            return self.fget(obj)
    def __set__(self, obj, value):
        raise AttributeError("Cannot set the attribute")
    def __delete__(self, obj):
        raise AttributeError("Cannot delete the attribute")

def _repr_argval(obj):
    """ Helper functions to display an R object in the docstring.
    This a hack and will be hopefully replaced the extraction of
    information from the R help system."""
    try:
        l = len(obj)
        if l == 1:
            if obj[0].rid == rinterface.MissingArg.rid:
                # no default value
                s = None
            elif obj[0].rid == rinterface.NULL.rid:
                s = 'rinterface.NULL'
            else:
                s = str(obj[0][0])
        elif l > 1:
            s = '(%s, ...)' % str(obj[0][0])
        else:
            s = str(obj)
    except:
        s = str(obj)
    return s

class Function(RObjectMixin, rinterface.SexpClosure):
    """ Python representation of an R function.
    """
    __local = baseenv_ri.get('local')
    __call = baseenv_ri.get('call')
    __assymbol = baseenv_ri.get('as.symbol')
    __newenv = baseenv_ri.get('new.env')

    _local_env = None
        
    def __init__(self, *args, **kwargs):
        super(Function, self).__init__(*args, **kwargs)
        self._local_env = self.__newenv(hash=rinterface.BoolSexpVector((True, )))

    @docstring_property(__doc__)
    def __doc__(self):
        fm = _formals_fixed(self)
        doc = list(['Python representation of an R function.',
                    'R arguments:', ''])
        if fm is rinterface.NULL:
            doc.append('<No information available>')
        for key, val in zip(fm.do_slot('names'), fm):
            if key == '...':
                val = 'R ellipsis (any number of parameters)'
            doc.append('%s: %s' % (key, _repr_argval(val)))
        return os.linesep.join(doc)


    def __call__(self, *args, **kwargs):
        new_args = [conversion.py2ri(a) for a in args]
        new_kwargs = {}
        for k, v in kwargs.items():
            new_kwargs[k] = conversion.py2ri(v)
        res = super(Function, self).__call__(*new_args, **new_kwargs)
        res = conversion.ri2ro(res)
        return res

    def formals(self):
        """ Return the signature of the underlying R function 
        (as the R function 'formals()' would).
        """
        res = _formals_fixed(self)
        res = conversion.ri2ro(res)
        return res

    def rcall(self, *args):
        """ Wrapper around the parent method rpy2.rinterface.SexpClosure.rcall(). """
        res = super(Function, self).rcall(*args)
        # XXX: Now that ri2ro converts to python objects, we restrict to the
        # default now for a raw R call. I'm not sure there should be *any*
        # conversion. However, we need to get this out of __init__.py first!
        # res = ro.default_ri2ro(res)
        return res

class SignatureTranslatedFunction(Function):
    """ Python representation of an R function, where
    the names in named argument are translated to valid
    argument names in Python. """
    _prm_translate = None


    def __init__(self, sexp,
                 init_prm_translate = None,
                 on_conflict = 'warn',
                 symbol_r2python = default_symbol_r2python,
                 symbol_check_after = default_symbol_check_after):
        super(SignatureTranslatedFunction, self).__init__(sexp)
        if init_prm_translate is None:
            prm_translate = OrderedDict()
        else:
            assert isinstance(init_prm_translate, dict)
            prm_translate = init_prm_translate

        formals = self.formals()
        if formals is not rinterface.NULL:
            (symbol_mapping, 
             conflicts, 
             resolutions) = _map_symbols(formals.names,
                                         translation = prm_translate,
                                         symbol_r2python = symbol_r2python,
                                         symbol_check_after = symbol_check_after)

            msg_prefix = 'Conflict when converting R symbols'+\
                ' in the function\'s signature:\n- '
            exception = ValueError
            _fix_map_symbols(symbol_mapping,
                             conflicts,
                             on_conflict,
                             msg_prefix,
                             exception)
            symbol_mapping.update(resolutions)
            reserved_pynames = set(dir(self))
            
            prm_translate.update((k, v[0]) for k, v in symbol_mapping.items())
        self._prm_translate = prm_translate
        if hasattr(sexp, '__rname__'):
            self.__rname__ = sexp.__rname__

    def __call__(self, *args, **kwargs):
        prm_translate = self._prm_translate
        for k in tuple(kwargs.keys()):
            r_k = prm_translate.get(k, None)
            if r_k is not None:
                v = kwargs.pop(k)
                kwargs[r_k] = v
        return super(SignatureTranslatedFunction, self).__call__(*args, **kwargs)

pattern_link = re.compile(r'\\link\{(.+?)\}')
pattern_code = re.compile(r'\\code\{(.+?)\}')
pattern_samp = re.compile(r'\\samp\{(.+?)\}')
class DocumentedSTFunction(SignatureTranslatedFunction):

    def __init__(self, sexp, init_prm_translate = None,
                 packagename = None):
        super(DocumentedSTFunction, 
              self).__init__(sexp, 
                             init_prm_translate = init_prm_translate)
        self.__rpackagename__ = packagename

    @docstring_property(__doc__)
    def __doc__(self):
        doc = ['Python representation of an R function.']
        description = help.docstring(self.__rpackagename__,
                                     self.__rname__,
                                     sections=['description'])
        doc.append(description)

        fm = _formals_fixed(self)
        names = fm.do_slot('names')
        doc.append(self.__rname__+'(')
        for key, val in self._prm_translate.items():
            if key == '___':
                description = '(was "..."). R ellipsis (any number of parameters)'
            else:
                description = _repr_argval(fm[names.index(val)])
            if description is None:
                doc.append('    %s,' % key)                
            else:
                doc.append('    %s = %s,' % (key, description))
        doc.extend((')', ''))
        package = help.Package(self.__rpackagename__)
        page = package.fetch(self.__rname__)
        for item in page.arguments():
            description = item.value
            description = description.replace('\n', '')
            description, count = pattern_link.subn(r'\1', description)
            description, count = pattern_code.subn(r'`\1`', description)
            description, count = pattern_samp.subn(r'`\1`', description)
            doc.append(' '.join((item.name, ': ', description, ',')))
            doc.append('')
        return os.linesep.join(doc)



import unittest
import pickle
import multiprocessing
import rpy2
import rpy2.rinterface as rinterface
import sys, os, subprocess, time, tempfile, io, signal, gc



if sys.version_info[0] == 3:
    IS_PYTHON3 = True
else:
    from itertools import izip as zip
    range = xrange
    IS_PYTHON3 = False

rinterface.initr()




def onlyAQUAorWindows(function):
    def res(self):
        platform = rinterface.baseenv.get('.Platform')
        platform_gui = [e for i, e in enumerate(platform.do_slot('names')) if e == 'GUI'][0]
        platform_ostype = [e for i, e in enumerate(platform.do_slot('names')) if e == 'OS.type'][0]
        if (platform_gui != 'AQUA') and (platform_ostype != 'windows'):
            self.assertTrue(False) # cannot be tested outside GUI==AQUA or OS.type==windows
            return None
        else:
            return function(self)

class CustomException(Exception):
    pass

class EmbeddedRTestCase(unittest.TestCase):


    def testConsolePrint(self):
        if sys.version_info[0] == 3:
            tmp_file = io.StringIO()
            stdout = sys.stdout
            sys.stdout = tmp_file
            try:
                rinterface.consolePrint('haha')
            except Exception as e:
                sys.stdout = stdout
                raise e
            sys.stdout = stdout
            tmp_file.flush()
            tmp_file.seek(0)
            self.assertEqual('haha', ''.join(s for s in tmp_file).rstrip())
            tmp_file.close()
        else:
            # no need to test which Python 2, only 2.7 supported
            with tempfile.NamedTemporaryFile() as tmp_file:
                stdout = sys.stdout
                sys.stdout = tmp_file
                try:
                    rinterface.consolePrint('haha')
                except Exception as e:
                    sys.stdout = stdout
                    raise e
                sys.stdout = stdout
                tmp_file.flush()
                tmp_file.seek(0)
                self.assertEqual('haha', ''.join(s.decode() for s in tmp_file))

    def testCallErrorWhenEndedR(self):
        if sys.version_info[0] == 2 and sys.version_info[1] < 6:
            self.assertTrue(False) # cannot be tested with Python < 2.6
            return None
        import multiprocessing
        def foo(queue):
            import rpy2.rinterface as rinterface
            rdate = rinterface.baseenv['date']
            rinterface.endr(1)
            try:
                tmp = rdate()
                res = (False, None)
            except RuntimeError as re:
                res = (True, re)
            except Exception as e:
                res = (False, e)
            queue.put(res)
        q = multiprocessing.Queue()
        p = multiprocessing.Process(target = foo, args = (q,))
        p.start()
        res = q.get()
        p.join()
        self.assertTrue(res[0])

    def testStr_typeint(self):
        t = rinterface.baseenv['letters']
        self.assertEqual('STRSXP', rinterface.str_typeint(t.typeof))
        t = rinterface.baseenv['pi']
        self.assertEqual('REALSXP', rinterface.str_typeint(t.typeof))

    def testStr_typeint_invalid(self):
        self.assertRaises(LookupError, rinterface.str_typeint, 99)

    def testGet_initoptions(self):
        options = rinterface.get_initoptions()
        self.assertEqual(len(rinterface.initoptions),
                          len(options))
        for o1, o2 in zip(rinterface.initoptions, options):
            self.assertEqual(o1, o2)
        
    def testSet_initoptions(self):
        self.assertRaises(RuntimeError, rinterface.set_initoptions, 
                          ('aa', '--verbose', '--no-save'))

    def testInitr(self):
        def init_r(preserve_hash):
            from rpy2 import rinterface
            rinterface.initr(r_preservehash=preserve_hash)
        preserve_hash = True
        proc = multiprocessing.Process(target=init_r,
                                       args=(preserve_hash,))
        proc.start()
        proc.join()

    def testParse(self):
        xp = rinterface.parse("2 + 3")
        self.assertEqual(rinterface.EXPRSXP, xp.typeof)
        self.assertEqual(2.0, xp[0][1][0])
        self.assertEqual(3.0, xp[0][2][0])

    def testParseUnicode(self):
        xp = rinterface.parse(u'"\u21a7"')
        self.assertEqual(1, len(xp))
        self.assertEqual(1, len(xp[0]))

    def testParseIncompleteError(self):
        self.assertRaises(rinterface.RParsingIncompleteError,
                          rinterface.parse, "2 + 3 /")

    def testParseError(self):
        self.assertRaises(rinterface.RParsingError,
                          rinterface.parse, "2 + 3 , 1")

    def testRternalize(self):
        def f(x, y):
            return x[0]+y[0]
        rfun = rinterface.rternalize(f)
        res = rfun(1, 2)
        self.assertEqual(3, res[0])

    def testRternalizeNamedArgs(self):
        def f(x, y, z=None):
            if z is None:
                return x[0]+y[0]
            else:
                return z
        rfun = rinterface.rternalize(f)
        res = rfun(1, 2)
        self.assertEqual(3, res[0])
        res = rfun(1, 2, z=8)
        self.assertEqual(8, res[0])

    def testExternalPython(self):
        def f(x):
            return 3

        rpy_fun = rinterface.SexpExtPtr(f, tag = rinterface.python_type_tag)
        _python = rinterface.StrSexpVector(('.Python', ))
        res = rinterface.baseenv['.External'](_python,
                                              rpy_fun, 1)
        self.assertEqual(3, res[0])
        self.assertEqual(1, len(res))

    def testExternalPythonFromExpression(self):
        xp_name = rinterface.StrSexpVector(('expression',))
        xp = rinterface.baseenv['vector'](xp_name, 3)
        
        

    def testParseInvalidString(self):
        self.assertRaises(ValueError, rinterface.parse, 3)

    def testInterruptR(self):
        if sys.version_info[0] == 2 and sys.version_info[1] < 6:
            self.assertTrue(False) # Test unit currently requires Python >= 2.6
        with tempfile.NamedTemporaryFile(mode = 'w', suffix = '.py',
                                         delete = False) as rpy_code:
            rpy2_path = os.path.dirname(rpy2.__path__[0])
            if IS_PYTHON3:
                pyexception_as = ' as'
            else:
                pyexception_as = ','

            rpy_code_str = """
import sys
sys.path.insert(0, '%s')
import rpy2.rinterface as ri

ri.initr()
def f(x):
    pass
ri.set_writeconsole_regular(f)
rcode = \"\"\"
i <- 0;
while(TRUE) {
    i <- i+1;
    Sys.sleep(0.01);
}\"\"\"
try:
  ri.baseenv['eval'](ri.parse(rcode))
except Exception%s e:
  sys.exit(0)
      """ %(rpy2_path, pyexception_as)

            rpy_code.write(rpy_code_str)
        cmd = (sys.executable, rpy_code.name)
        with open(os.devnull, 'w') as fnull:
            child_proc = subprocess.Popen(cmd,
                                          stdout=fnull,
                                          stderr=fnull)
            time.sleep(1)  # required for the SIGINT to function
            # (appears like a bug w/ subprocess)
            # (the exact sleep time migth be machine dependent :( )
            child_proc.send_signal(signal.SIGINT)
            time.sleep(1)  # required for the SIGINT to function
            ret_code = child_proc.poll()
        self.assertFalse(ret_code is None) # Interruption failed

    def testRpyMemory(self):
        x = rinterface.SexpVector(range(10), rinterface.INTSXP)
        y = rinterface.SexpVector(range(10), rinterface.INTSXP)
        x_rid = x.rid
        self.assertTrue(x_rid in set(z[0] for z in rinterface.protected_rids()))
        del(x)
        gc.collect(); gc.collect()
        self.assertFalse(x_rid in set(z[0] for z in rinterface.protected_rids()))

class CallbacksTestCase(unittest.TestCase):
    def tearDown(self):
        rinterface.set_writeconsole_regular(rinterface.consolePrint)
        rinterface.set_readconsole(rinterface.consoleRead)
        rinterface.set_readconsole(rinterface.consoleFlush)
        rinterface.set_choosefile(rinterface.chooseFile)
        sys.last_value = None

    def testSetWriteConsoleRegular(self):
        buf = []
        def f(x):
            buf.append(x)

        rinterface.set_writeconsole_regular(f)
        self.assertEqual(rinterface.get_writeconsole_regular(), f)
        code = rinterface.SexpVector(["3", ], rinterface.STRSXP)
        rinterface.baseenv["print"](code)
        self.assertEqual('[1] "3"\n', str.join('', buf))

    def testWriteConsoleRegularWithError(self):
        def f(x):
            raise CustomException("Doesn't work.")
        rinterface.set_writeconsole_regular(f)

        with tempfile.NamedTemporaryFile() as tmp_file:
            stderr = sys.stderr
            sys.stderr = tmp_file
            try:
                code = rinterface.SexpVector(["3", ], rinterface.STRSXP)
                rinterface.baseenv["print"](code)
            except Exception as e:
                sys.stderr = stderr
                raise e
            sys.stderr = stderr
            tmp_file.flush()
            tmp_file.seek(0)
            self.assertEqual("Doesn't work.", str(sys.last_value))
            #errorstring = ''.join(tmp_file.readlines())
            #self.assertTrue(errorstring.startswith('Traceback'))

    def testSetResetConsole(self):
        reset = [0]
        def f():
            reset[0] += 1

        rinterface.set_resetconsole(f)
        self.assertEqual(rinterface.get_resetconsole(), f)
        try:
            rinterface.baseenv['eval'](rinterface.parse('1+"a"'))
        except rinterface.RRuntimeError:
            pass
        self.assertEqual(1, reset[0])

    @onlyAQUAorWindows
    def testSetFlushConsole(self):
        flush = {'count': 0}
        def f():
            flush['count'] = flush['count'] + 1

        rinterface.set_flushconsole(f)
        self.assertEqual(rinterface.get_flushconsole(), f)
        rinterface.baseenv.get("flush.console")()
        self.assertEqual(1, flush['count'])
        rinterface.set_writeconsole_regular(rinterface.consoleFlush)

    @onlyAQUAorWindows
    def testFlushConsoleWithError(self):
        def f(prompt):
            raise Exception("Doesn't work.")
        rinterface.set_flushconsole(f)

        with tempfile.NamedTemporaryFile() as tmp_file:
            stderr = sys.stderr
            sys.stderr = tmp_file
            try:
                res = rinterface.baseenv.get("flush.console")()
            except Exception as e:
                sys.stderr = stderr
                raise e
            sys.stderr = stderr
            tmp_file.flush()
            tmp_file.seek(0)
            self.assertEqual("Doesn't work.", str(sys.last_value))
            #errorstring = ''.join(tmp_file.readlines())
            #self.assertTrue(errorstring.startswith('Traceback'))

    def testSetReadConsole(self):
        yes = "yes\n"
        def sayyes(prompt):
            return yes
        rinterface.set_readconsole(sayyes)
        self.assertEqual(rinterface.get_readconsole(), sayyes)
        res = rinterface.baseenv["readline"]()
        self.assertEqual(yes.strip(), res[0])
        rinterface.set_readconsole(rinterface.consoleRead)

    def testReadConsoleWithError(self):
        def f(prompt):
            raise Exception("Doesn't work.")
        rinterface.set_readconsole(f)

        with tempfile.NamedTemporaryFile() as tmp_file:

            stderr = sys.stderr
            sys.stderr = tmp_file
            try:
                res = rinterface.baseenv["readline"]()
            except Exception as e:
                sys.stderr = stderr
                raise e
            sys.stderr = stderr
            tmp_file.flush()
            tmp_file.seek(0)
            self.assertEqual("Doesn't work.", str(sys.last_value))
            #errorstring = ''.join(tmp_file.readlines())
            #self.assertTrue(errorstring.startswith('Traceback'))
        
    def testSetShowMessage(self):
        def f(message):
            return "foo"
        rinterface.set_showmessage(f)
        #FIXME: incomplete test

    def testShowMessageWithError(self):
        def f(prompt):
            raise Exception("Doesn't work.")
        rinterface.set_showmessage(f)
        #FIXME: incomplete test

    def testSetChooseFile(self):
        me = "me"
        def chooseMe(prompt):
            return me
        rinterface.set_choosefile(chooseMe)
        self.assertEqual(rinterface.get_choosefile(), chooseMe)
        res = rinterface.baseenv["file.choose"]()
        self.assertEqual(me, res[0])
        rinterface.set_choosefile(rinterface.chooseFile)

    def testChooseFileWithError(self):
        def noconsole(x):
            pass
        rinterface.set_writeconsole_regular(noconsole) # reverted by the tearDown method
        def f(prompt):
            raise Exception("Doesn't work.")
        rinterface.set_choosefile(f)
        self.assertRaises(rinterface.RRuntimeError,
                          rinterface.baseenv["file.choose"])
        self.assertEqual("Doesn't work.", str(sys.last_value))

    def testSetShowFiles(self):
        sf = []
        def f(fileheaders, wtitle, fdel, pager):
            sf.append(wtitle)
            for tf in fileheaders:
                sf.append(tf)

        rinterface.set_showfiles(f)
        file_path = rinterface.baseenv["file.path"]
        r_home = rinterface.baseenv["R.home"]
        filename = file_path(r_home(rinterface.StrSexpVector(("doc", ))), 
                             rinterface.StrSexpVector(("COPYRIGHTS", )))
        res = rinterface.baseenv["file.show"](filename)
        self.assertEqual(filename[0], sf[1][1])
        self.assertEqual('R Information', sf[0])

    def testShowFilesWithError(self):
        def f(fileheaders, wtitle, fdel, pager):
            raise Exception("Doesn't work.")

        rinterface.set_showfiles(f)
        file_path = rinterface.baseenv["file.path"]
        r_home = rinterface.baseenv["R.home"]
        filename = file_path(r_home(rinterface.StrSexpVector(("doc", ))), 
                             rinterface.StrSexpVector(("COPYRIGHTS", )))

        with tempfile.NamedTemporaryFile() as tmp_file:
            stderr = sys.stderr
            sys.stderr = tmp_file
            try:
                res = rinterface.baseenv["file.show"](filename)
            except rinterface.RRuntimeError:
                pass
            except Exception as e:
                sys.stderr = stderr
                raise e
            sys.stderr = stderr
            tmp_file.flush()
            tmp_file.seek(0)
            self.assertEqual("Doesn't work.", str(sys.last_value))
            #errorstring = ''.join(tmp_file.readlines())
            #self.assertTrue(errorstring.startswith('Traceback'))

    def testSetCleanUp(self):
        orig_cleanup = rinterface.get_cleanup()
        def f(saveact, status, runlast):
            return False
        rinterface.set_cleanup(f)
        rinterface.set_cleanup(orig_cleanup)

    def testCleanUp(self):
        orig_cleanup = rinterface.get_cleanup()
        def f(saveact, status, runlast):
            return None
        r_quit = rinterface.baseenv['q']
        rinterface.set_cleanup(f)        
        self.assertRaises(rinterface.RRuntimeError, r_quit)
        rinterface.set_cleanup(orig_cleanup)


class ObjectDispatchTestCase(unittest.TestCase):
    def testObjectDispatchLang(self):
        formula = rinterface.globalenv.get('formula')
        obj = formula(rinterface.StrSexpVector(['y ~ x', ]))
        self.assertTrue(isinstance(obj, rinterface.SexpVector))
        self.assertEqual(rinterface.LANGSXP, obj.typeof)

    def testObjectDispatchVector(self):
        letters = rinterface.globalenv.get('letters')
        self.assertTrue(isinstance(letters, rinterface.SexpVector))

    def testObjectDispatchClosure(self):
        #import pdb; pdb.set_trace()
        help = rinterface.globalenv.get('sum')
        self.assertTrue(isinstance(help, rinterface.SexpClosure))

    def testObjectDispatchRawVector(self):
        raw = rinterface.baseenv.get('raw')
        #rawvec = raw(rinterface.IntSexpVector((10, )))
        #self.assertEqual(rinterface.RAWSXP, rawvec.typeof)

class SerializeTestCase(unittest.TestCase):
    def testUnserialize(self):
        x = rinterface.IntSexpVector([1,2,3])
        x_serialized = x.__getstate__()
        x_again = rinterface.unserialize(x_serialized, x.typeof)
        identical = rinterface.baseenv["identical"]
        self.assertFalse(x.rsame(x_again))
        self.assertTrue(identical(x, x_again)[0])

    def testPickle(self):
        x = rinterface.IntSexpVector([1,2,3])
        with tempfile.NamedTemporaryFile() as f:
            pickle.dump(x, f)
            f.flush()
            f.seek(0)
            x_again = pickle.load(f)
        identical = rinterface.baseenv["identical"]
        self.assertTrue(identical(x, x_again)[0])
                     
def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(EmbeddedRTestCase)
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(CallbacksTestCase))
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(ObjectDispatchTestCase))
    return suite

if __name__ == '__main__':
    tr = unittest.TextTestRunner(verbosity = 2)
    tr.run(suite())
    



"""
Wrapper for the popular R library ggplot2.

With rpy2, the most convenient general way to
import packages is to use `importr()`, for example
with ggplot2::

    from rpy2.robjects.packages import importr
    ggplot2 = importr('ggplot2')

This module is an supplementary layer in which an attempt
at modelling the package as it was really developed
as Python package is made. Behind the scene, `importr()`
is used and can be accessed with:

    from robjects.robjects.lib import ggplot2
    ggplot2.ggplot2

GGplot2 is designed using a prototype-based approach to
Object-Oriented Programming, and this module is trying
to define class-hierachies so the nature of a given
instance can be identified more easily.

The main families of classes are:

- GGplot
- Aes and AesString
- Layer
- Stat

A downside of the approach is that the code in the
module is 'hand-made'. In hindsight, this can be tedious
to maintain and document but this is a good showcase of
"manual" mapping of R code into Python classes.

The codebase in R for ggplot2 has evolved since this
was initially written, and many functions have
signature-defined parameters (used to be ellipsis
about everywhere). Metaprogramming will hopefully
be added to shorten the Python code in the module,
and provide a more dynamic mapping.

"""


import rpy2.robjects as robjects
import rpy2.robjects.conversion as conversion
import rpy2.rinterface as rinterface
from rpy2.robjects.packages import importr, WeakPackage
import copy
import warnings

NULL = robjects.NULL

ggplot2 = importr('ggplot2', on_conflict="warn")
ggplot2 = WeakPackage(ggplot2._env,
                      ggplot2.__rname__,
                      translation=ggplot2._translation,
                      exported_names=ggplot2._exported_names,
                      on_conflict="warn",
                      version=ggplot2.__version__,
                      symbol_r2python=ggplot2._symbol_r2python,
                      symbol_check_after=ggplot2._symbol_check_after)

TARGET_VERSION = '2.2.0'
if ggplot2.__version__ != TARGET_VERSION:
   warnings.warn('This was designed againt ggplot2 version %s but you have %s' % (TARGET_VERSION, ggplot2.__version__))
ggplot2_env = robjects.baseenv['as.environment']('package:ggplot2')

StrVector = robjects.StrVector

def as_symbol(x):
    res = rinterface.parse(x)
    return res

class GGPlot(robjects.RObject):
    """ A Grammar of Graphics Plot.
    
    GGPlot instances can be added to one an other in order to construct
    the final plot (the method `__add__()` is implemented).
    """

    _constructor = ggplot2._env['ggplot']
    _rprint = ggplot2._env['print.ggplot']
    _add = ggplot2._env['%+%']

    @classmethod
    def new(cls, data):
        """ Constructor for the class GGplot. """
        data = conversion.py2ri(data)
        res = cls(cls._constructor(data))
        return res
    
    def plot(self, vp = robjects.constants.NULL):
        self._rprint(self, vp = vp)

    def __add__(self, obj):
        res = self._add(self, obj)
        if res.rclass[0] != 'gg':
            raise ValueError("Added object did not give a ggplot result (get class '%s')." % res.rclass[0])
        return self.__class__(res)

    def save(self, filename, **kwargs):
        """ Save the plot ( calls R's `ggplot2::ggsave()` ) """
        ggplot2.ggsave(filename=filename, plot=self, **kwargs)

ggplot = GGPlot.new


class Aes(robjects.Vector):
    """ Aesthetics mapping, using expressions rather than string
    (this is the most common form when using the package in R - it might
    be easier to use AesString when working in Python using rpy2 -
    see class AesString in this Python module).
    """
    _constructor = ggplot2_env['aes']
    
    @classmethod
    def new(cls, **kwargs):
        """Constructor for the class Aes."""
        new_kwargs = copy.copy(kwargs)
        for k,v in kwargs.items():
            new_kwargs[k] = as_symbol(v)
        res = cls(cls._constructor(**new_kwargs))
        return res
aes = Aes.new

class AesString(robjects.Vector):
    """ Aesthetics mapping, using strings rather than expressions (the later
    being most common form when using the package in R - see class Aes
    in this Python module).
    
    This associates dimensions in the data sets (columns in the DataFrame),
    possibly with a transformation applied on-the-fly (e.g., "log(value)",
    or "cost / benefits") to graphical "dimensions" in a chosen graphical
    representation (e.g., x-axis, color of points, size, etc...).
    
    Not all graphical representations have all dimensions. Refer to the
    documentation of ggplot2, online tutorials, or Hadley's book for
    more details.
    """
    _constructor = ggplot2_env['aes_string']
    
    @classmethod
    def new(cls, **kwargs):
       """Constructor for the class AesString."""
       res = cls(cls._constructor(**kwargs))
       return res
aes_string = AesString.new


class Layer(robjects.RObject):
     """ At this level, aesthetics mapping can (should ?) be specified
     (see Aes and AesString). """
     _constructor = ggplot2_env['layer']
    #_dollar = proto_env["$.proto"]

     @classmethod
     def new(cls,
            *args, **kwargs):
        """ Constructor for the class Layer. """
        for i, elt in enumerate(args):
            args[i] = conversion.py2ro(elt)

        for k in kwargs:
           kwargs[k] = conversion.py2ro(kwargs[k])
          
        res = cls(cls.contructor)(*args, **kwargs)
        return res

layer = Layer.new        

class GBaseObject(robjects.RObject):
    @classmethod
    def new(*args, **kwargs):
        args_list = list(args)
        cls = args_list.pop(0)
        res = cls(cls._constructor(*args_list, **kwargs))
        return res

class Stat(GBaseObject):
    """ A "statistical" processing of the data in order
    to make a plot, or a plot element.
    
    This is an abstract class; material classes are called
    Stat* (e.g., StatAbline, StatBin, etc...). """
    pass


class StatBin(Stat):
    """ Bin data. """
    _constructor = ggplot2_env['stat_bin']
stat_bin = StatBin.new

class StatBin2D(Stat):
    """ 2D binning of data into squares/rectangles. """
    try:
       _constructor = ggplot2_env['stat_bin_2d']
    except:
       # fallback for versions of ggplot2 < 2.0
       _constructor = ggplot2_env['stat_bin2d']
stat_bin2d = StatBin2D.new
stat_bin_2d=StatBin2D.new

class StatBinhex(Stat):
    """ 2D binning of data into hexagons. """
    try:
       _constructor = ggplot2_env['stat_bin_hex']
    except:
       # fallback for versions of ggplot2 < 2.0
       _constructor = ggplot2_env['stat_binhex']
stat_binhex = StatBinhex.new
stat_bin_hex=StatBinhex.new

class StatBoxplot(Stat):
    """ Components of box and whisker plot. """
    _constructor = ggplot2_env['stat_boxplot']
stat_boxplot = StatBoxplot.new

class StatContour(Stat):
    """ Contours of 3D data. """
    _constructor = ggplot2_env['stat_contour']
stat_contour = StatContour.new

class StatDensity(Stat):
    """ 1D density estimate """
    _constructor = ggplot2_env['stat_density']
stat_density = StatDensity.new

class StatDensity2D(Stat):
    """ 2D density estimate """
    try:
       _constructor = ggplot2_env['stat_density_2d']
    except:
       _constructor = ggplot2_env['stat_density2d']
stat_density2d = StatDensity2D.new
stat_density_2d=StatDensity2D.new

class StatFunction(Stat):
    """ Superimpose a function """
    _constructor = ggplot2_env['stat_function']
stat_function = StatFunction.new

class StatIdentity(Stat):
    """ Identity function """
    _constructor = ggplot2_env['stat_identity']
stat_identity = StatIdentity.new

class StatQQ(Stat):
    """ Calculation for quantile-quantile plot. """
    _constructor = ggplot2_env['stat_qq']
stat_qq = StatQQ.new

class StatQuantile(Stat):
    """ Continuous quantiles """
    _constructor = ggplot2_env['stat_quantile']
stat_quantile = StatQuantile.new

class StatSmooth(Stat):
    """ Smoothing function """
    _constructor = ggplot2_env['stat_smooth']
stat_smooth = StatSmooth.new

class StatSpoke(Stat):
    """ Convert angle and radius to xend and yend """
    _constructor = ggplot2_env['stat_spoke']
stat_spoke = StatSpoke.new

class StatSum(Stat):
    """ Sum unique values.
    Useful when overplotting. """
    _constructor = ggplot2_env['stat_sum']
stat_sum = StatSum.new

class StatSummary(Stat):
    """ Summarize values for y at every unique value for x"""
    _constructor = ggplot2_env['stat_summary']
stat_summary = StatSummary.new

class StatSummary2D(Stat):
    """ Summarize values for y at every unique value for x"""
    try:
       _constructor = ggplot2_env['stat_summary_2d']
    except:
       _constructor = ggplot2_env['stat_summary2d']
stat_summary2d = StatSummary2D.new
stat_summary_2d = StatSummary2D.new

class StatUnique(Stat):
    """ Remove duplicates. """
    _constructor = ggplot2_env['stat_unique']
stat_unique = StatUnique.new

class Coord(GBaseObject):
    """ Coordinates """
    pass

class CoordFixed(Coord):
    """ Cartesian coordinates with fixed relationship
    (that is fixed ratio between units in axes).
    CoordEquel seems to be identical to this class."""
    _constructor = ggplot2_env['coord_fixed']
coord_fixed = CoordFixed.new

class CoordCartesian(Coord):
    """ Cartesian coordinates. """
    _constructor = ggplot2_env['coord_cartesian']
coord_cartesian = CoordCartesian.new

class CoordEqual(Coord):
    """ This class seems to be identical to CoordFixed. """
    _constructor = ggplot2_env['coord_equal']
coord_equal = CoordEqual.new

class CoordFlip(Coord):
    """ Flip horizontal and vertical coordinates. """
    _constructor = ggplot2_env['coord_flip']
coord_flip = CoordFlip.new

class CoordMap(Coord):
    """ Map projections. """
    _constructor = ggplot2_env['coord_map']
coord_map = CoordMap.new

class CoordPolar(Coord):
    """ Polar coordinates. """
    _constructor = ggplot2_env['coord_polar']
coord_polar = CoordPolar.new

class CoordTrans(Coord):
    """ Apply transformations (functions) to a cartesian coordinate system. """
    _constructor = ggplot2_env['coord_trans']
coord_trans = CoordTrans.new


class Facet(GBaseObject):
    """ Panels """
    pass

class FacetGrid(Facet):
    """ Panels in a grid. """
    _constructor = ggplot2_env['facet_grid']
facet_grid = FacetGrid.new

class FacetWrap(Facet):
    """ Sequence of panels in a 2D layout """
    _constructor = ggplot2_env['facet_wrap']
facet_wrap = FacetWrap.new

class Geom(GBaseObject):
    pass
       
class GeomAbline(Geom):
    _constructor = ggplot2_env['geom_abline']
geom_abline = GeomAbline.new

class GeomArea(Geom):
    _constructor = ggplot2_env['geom_area']
geom_area = GeomArea.new

class GeomBar(Geom):
    _constructor = ggplot2_env['geom_bar']
geom_bar = GeomBar.new

class GeomBin2D(Geom):
    _constructor = ggplot2_env['geom_bin2d']
geom_bin2d = GeomBin2D.new

class GeomBlank(Geom):
    _constructor = ggplot2_env['geom_blank']
geom_blank = GeomBlank.new
   
class GeomBoxplot(Geom):
    _constructor = ggplot2_env['geom_boxplot']
geom_boxplot = GeomBoxplot.new
   
class GeomContour(Geom):
    _constructor = ggplot2_env['geom_contour']
geom_contour = GeomContour.new

class GeomCrossBar(Geom):
    _constructor = ggplot2_env['geom_crossbar']
geom_crossbar = GeomCrossBar.new

class GeomDensity(Geom):
    _constructor = ggplot2_env['geom_density']
geom_density = GeomDensity.new

class GeomDensity2D(Geom):
    try:
       _constructor = ggplot2_env['geom_density_2d']
    except:
       _constructor = ggplot2_env['geom_density2d']
geom_density2d = GeomDensity2D.new
geom_density_2d = GeomDensity2D.new

class GeomDotplot(Geom):
   _constructor = ggplot2_env['geom_dotplot']
geom_dotplot = GeomDotplot.new
   
class GeomErrorBar(Geom):
    _constructor = ggplot2_env['geom_errorbar']
geom_errorbar = GeomErrorBar.new

class GeomErrorBarH(Geom):
    _constructor = ggplot2_env['geom_errorbarh']
geom_errorbarh = GeomErrorBarH.new

class GeomFreqPoly(Geom):
    _constructor = ggplot2_env['geom_freqpoly']
geom_freqpoly = GeomFreqPoly.new

class GeomHex(Geom):
    _constructor = ggplot2_env['geom_hex']
geom_hex = GeomHex.new

class GeomHistogram(Geom):
    _constructor = ggplot2_env['geom_histogram']
geom_histogram = GeomHistogram.new

class GeomHLine(Geom):
    _constructor = ggplot2_env['geom_hline']
geom_hline = GeomHLine.new

class GeomJitter(Geom):
    _constructor = ggplot2_env['geom_jitter']
geom_jitter = GeomJitter.new

class GeomLine(Geom):
    _constructor = ggplot2_env['geom_line']
geom_line = GeomLine.new

class GeomLineRange(Geom):
    _constructor = ggplot2_env['geom_linerange']
geom_linerange = GeomLineRange.new
   
class GeomPath(Geom):
    _constructor = ggplot2_env['geom_path']
geom_path = GeomPath.new

class GeomPoint(Geom):
    _constructor = ggplot2_env['geom_point']
geom_point = GeomPoint.new

class GeomPointRange(Geom):
    _constructor = ggplot2_env['geom_pointrange']
geom_pointrange = GeomPointRange.new

class GeomPolygon(Geom):
    _constructor = ggplot2_env['geom_polygon']
geom_polygon = GeomPolygon.new

class GeomQuantile(Geom):
    _constructor = ggplot2_env['geom_quantile']
geom_quantile = GeomQuantile.new

class GeomRaster(Geom):
    _constructor = ggplot2_env['geom_raster']
geom_raster = GeomRaster.new

class GeomRect(Geom):
    _constructor = ggplot2_env['geom_rect']
geom_rect = GeomRect.new

class GeomRibbon(Geom):
    _constructor = ggplot2_env['geom_ribbon']
geom_ribbon = GeomRibbon.new

class GeomRug(Geom):
    _constructor = ggplot2_env['geom_rug']
geom_rug = GeomRug.new

class GeomSegment(Geom):
    _constructor = ggplot2_env['geom_segment']
geom_segment = GeomSegment.new

class GeomSmooth(Geom):
    _constructor = ggplot2_env['geom_smooth']
geom_smooth = GeomSmooth.new

class GeomSpoke(Geom):
    """ Convert angle and radius to xend and yend """
    _constructor = ggplot2_env['geom_spoke']
geom_spoke = GeomSpoke.new

class GeomStep(Geom):
    _constructor = ggplot2_env['geom_step']
geom_step = GeomStep.new

class GeomText(Geom):
    _constructor = ggplot2_env['geom_text']
geom_text = GeomText.new

class GeomTile(Geom):
    _constructor = ggplot2_env['geom_tile']
geom_tile = GeomTile.new

class GeomVLine(Geom):
    _constructor = ggplot2_env['geom_vline']
geom_vline = GeomVLine.new

class Position(GBaseObject):
    pass

class PositionDodge(Position):
    _constructor = ggplot2_env['position_dodge']
position_dodge = PositionDodge.new
class PositionFill(Position):
    _constructor = ggplot2_env['position_fill']
position_fill = PositionFill.new
class PositionJitter(Position):
    _constructor = ggplot2_env['position_jitter']
position_jitter = PositionJitter.new
class PositionStack(Position):
    _constructor = ggplot2_env['position_stack']
position_stack = PositionStack.new

class Scale(GBaseObject):
    pass

class ScaleAlpha(Scale):
    _constructor = ggplot2_env['scale_alpha']
scale_alpha = ScaleAlpha.new
class ScaleColour(Scale):
    pass
class ScaleDiscrete(Scale):
    pass
class ScaleLinetype(Scale):
    _constructor = ggplot2_env['scale_linetype']
scale_linetype = ScaleLinetype.new
class ScaleShape(Scale):
    _constructor = ggplot2_env['scale_shape']
scale_shape = ScaleShape.new
class ScaleSize(Scale):
    _constructor = ggplot2_env['scale_size']
scale_size = ScaleSize.new

class ScaleShapeDiscrete(Scale):
   _constructor = ggplot2_env['scale_shape_discrete']
scale_shape_discrete = ScaleShapeDiscrete.new

class ScaleFill(Scale):
    pass
class ScaleX(Scale):
    pass
class ScaleY(Scale):
    pass

# class Limits(Scale):
#    _constructor = ggplot2_env['limits']
# limits = Limits.new

class XLim(Scale):
    _constructor = ggplot2_env['xlim']
xlim = XLim.new

class YLim(Scale):
    _constructor = ggplot2_env['ylim']
ylim = YLim.new


class ScaleXContinuous(ScaleX):
    _constructor = ggplot2_env['scale_x_continuous']
scale_x_continuous = ScaleXContinuous.new
class ScaleYContinuous(ScaleY):
    _constructor = ggplot2_env['scale_y_continuous']
scale_y_continuous = ScaleYContinuous.new
class ScaleXDiscrete(ScaleX):
    _constructor = ggplot2_env['scale_x_discrete']
scale_x_discrete = ScaleXDiscrete.new
class ScaleYDiscrete(ScaleY):
    _constructor = ggplot2_env['scale_y_discrete']
scale_y_discrete = ScaleYDiscrete.new
class ScaleXDate(ScaleX):
    _constructor = ggplot2_env['scale_x_date']
scale_x_date = ScaleXDate.new
class ScaleYDate(ScaleY):
    _constructor = ggplot2_env['scale_y_date']
scale_y_date = ScaleYDate.new
class ScaleXDatetime(ScaleX):
    _constructor = ggplot2_env['scale_x_datetime']
scale_x_datetime = ScaleXDatetime.new
class ScaleYDatetime(ScaleY):
    _constructor = ggplot2_env['scale_y_datetime']
scale_y_datetime = ScaleYDatetime.new
class ScaleXLog10(ScaleX):
    _constructor = ggplot2_env['scale_x_log10']
scale_x_log10 = ScaleXLog10.new
class ScaleYLog10(ScaleY):
    _constructor = ggplot2_env['scale_y_log10']
scale_y_log10 = ScaleYLog10.new
class ScaleXReverse(ScaleX):
    _constructor = ggplot2_env['scale_x_reverse']
scale_x_reverse = ScaleXReverse.new
class ScaleYReverse(ScaleY):
    _constructor = ggplot2_env['scale_y_reverse']
scale_y_reverse = ScaleYReverse.new
class ScaleXSqrt(ScaleX):
    _constructor = ggplot2_env['scale_x_sqrt']
scale_x_sqrt = ScaleXSqrt.new
class ScaleYSqrt(ScaleY):
    _constructor = ggplot2_env['scale_y_sqrt']
scale_y_sqrt = ScaleYSqrt.new

class ScaleColourBrewer(ScaleColour):
    _constructor = ggplot2_env['scale_colour_brewer']
scale_colour_brewer = ScaleColourBrewer.new
scale_color_brewer = scale_colour_brewer

class ScaleColourContinuous(ScaleColour):
    _constructor = ggplot2_env['scale_colour_continuous']
scale_colour_continuous = ScaleColourContinuous.new
scale_color_continuous = scale_colour_continuous

class ScaleColourDiscrete(ScaleColour):
    _constructor = ggplot2_env['scale_colour_discrete']
scale_colour_discrete = ScaleColourDiscrete.new
scale_color_discrete = scale_colour_discrete

class ScaleColourGradient(ScaleColour):
    _constructor = ggplot2_env['scale_colour_gradient']
scale_colour_gradient = ScaleColourGradient.new
scale_color_gradient = scale_colour_gradient

class ScaleColourGradient2(ScaleColour):
    _constructor = ggplot2_env['scale_colour_gradient2']
scale_colour_gradient2 = ScaleColourGradient2.new
scale_color_gradient2 = scale_colour_gradient2

class ScaleColourGradientN(ScaleColour):
    _constructor = ggplot2_env['scale_colour_gradientn']
scale_colour_gradientn = ScaleColourGradientN.new
scale_color_gradientn = scale_colour_gradientn

class ScaleColourGrey(ScaleColour):
    _constructor = ggplot2_env['scale_colour_grey']
scale_colour_grey = ScaleColourGrey.new
scale_color_grey = scale_colour_grey

class ScaleColourHue(ScaleColour):
    _constructor = ggplot2_env['scale_colour_hue']
scale_colour_hue = ScaleColourHue.new
scale_color_hue = scale_colour_hue

class ScaleColourIdentity(ScaleColour):
    _constructor = ggplot2_env['scale_colour_identity']
scale_colour_identity = ScaleColourIdentity.new
scale_color_identity = scale_colour_identity

class ScaleColourManual(ScaleColour):
    _constructor = ggplot2_env['scale_colour_manual']
scale_colour_manual = ScaleColourManual.new
scale_color_manual = scale_colour_manual

class ScaleFillBrewer(ScaleFill):
    _constructor = ggplot2_env['scale_fill_brewer']
scale_fill_brewer = ScaleFillBrewer.new
class ScaleFillContinuous(ScaleFill):
    _constructor = ggplot2_env['scale_fill_continuous']
scale_fill_continuous = ScaleFillContinuous.new
class ScaleFillDiscrete(ScaleFill):
    _constructor = ggplot2_env['scale_fill_discrete']
scale_fill_discrete = ScaleFillDiscrete.new
class ScaleFillGradient(ScaleFill):
    _constructor = ggplot2_env['scale_fill_gradient']
scale_fill_gradient = ScaleFillGradient.new
class ScaleFillGradient2(ScaleFill):
    _constructor = ggplot2_env['scale_fill_gradient2']
scale_fill_gradient2 = ScaleFillGradient2.new
class ScaleFillGradientN(ScaleFill):
    _constructor = ggplot2_env['scale_fill_gradientn']
scale_fill_gradientn = ScaleFillGradientN.new
class ScaleFillGrey(ScaleFill):
     _constructor = ggplot2_env['scale_fill_grey']
scale_fill_grey = ScaleFillGrey.new
class ScaleFillHue(ScaleFill):
    _constructor = ggplot2_env['scale_fill_hue']
scale_fill_hue = ScaleFillHue.new
class ScaleFillIdentity(ScaleFill):
    _constructor = ggplot2_env['scale_fill_identity']
scale_fill_identity = ScaleFillIdentity.new
class ScaleFillManual(ScaleFill):
    _constructor = ggplot2_env['scale_fill_manual']
scale_fill_manual = ScaleFillManual.new
class ScaleLinetypeContinuous(ScaleLinetype):
    _constructor = ggplot2_env['scale_linetype_continuous']
scale_linetype_continuous = ScaleLinetypeContinuous.new
class ScaleLinetypeDiscrete(ScaleLinetype):
    _constructor = ggplot2_env['scale_linetype_discrete']
scale_linetype_discrete = ScaleLinetypeDiscrete.new
class ScaleLinetypeManual(ScaleLinetype):
    _constructor = ggplot2_env['scale_linetype_manual']
scale_linetype_manual = ScaleLinetypeManual.new
class ScaleShapeManual(ScaleShape):
    _constructor = ggplot2_env['scale_shape_manual']
scale_shape_manual = ScaleShapeManual.new



guides = ggplot2.guides
guide_colorbar = ggplot2.guide_colorbar
guide_colourbar = ggplot2.guide_colourbar
guide_legend = ggplot2.guide_legend

class Options(robjects.Vector):
   def __init__(self, obj):
      self.__sexp__ = obj.__sexp__

   def __repr__(self):
      s = '<instance of %s : %i>' %(type(self), id(self)) 
      return s


class Element(Options):
   pass

class ElementText(Element):
    _constructor = ggplot2.element_text
    @classmethod
    def new(cls, family = "", face = "plain", colour = "black", size = 10,
            hjust = 0.5, vjust = 0.5, angle = 0, lineheight = 1.1, 
            color = NULL):
       res = cls(cls._constructor(family = family, face = face, 
                                  colour = colour, size = size,
                                  hjust = hjust, vjust = vjust, 
                                  angle = angle, lineheight = lineheight))
       return res
element_text = ElementText.new

class ElementRect(Element):
    _constructor = ggplot2.element_rect
    @classmethod
    def new(cls, fill = NULL, colour = NULL, size = NULL,
            linetype = NULL, color = NULL):
       res = cls(cls._constructor(fill = fill, colour = colour,
                                  size = size, linetype = linetype,
                                  color = color))
       return res
element_rect = ElementRect.new

class Labs(Options):
   _constructor = ggplot2.labs
   @classmethod
   def new(cls, **kwargs):
      res = cls(cls._constructor(**kwargs))
      return res

labs = Labs.new

class Theme(Options):
   pass

class ElementBlank(Theme):
    _constructor = ggplot2.element_blank
    @classmethod
    def new(cls):
        res = cls(cls._constructor())
        return res

element_blank = ElementBlank.new

theme_get = ggplot2.theme_get

class ThemeGrey(Theme):
    _constructor = ggplot2.theme_grey
    @classmethod
    def new(cls, base_size = 12):
       res = cls(cls._constructor(base_size = base_size))
       return res

theme_grey = ThemeGrey.new

class ThemeClassic(Theme):
    _constructor = ggplot2.theme_classic
    @classmethod
    def new(cls, base_size = 12, base_family = ""):
       res = cls(cls._constructor(base_size = base_size,
                                  base_family = base_family))
       return res

theme_classic = ThemeClassic.new

class ThemeDark(Theme):
    _constructor = ggplot2.theme_dark
    @classmethod
    def new(cls, base_size = 12, base_family = ""):
       res = cls(cls._constructor(base_size = base_size,
                                  base_family = base_family))
       return res

theme_dark = ThemeDark.new

class ThemeLight(Theme):
    _constructor = ggplot2.theme_light
    @classmethod
    def new(cls, base_size = 12, base_family = ""):
       res = cls(cls._constructor(base_size = base_size,
                                  base_family = base_family))
       return res

theme_light = ThemeLight.new

class ThemeBW(Theme):
    _constructor = ggplot2.theme_bw
    @classmethod
    def new(cls, base_size = 12):
       res = cls(cls._constructor(base_size = base_size))
       return res

theme_bw = ThemeBW.new

class ThemeGray(Theme):
    _constructor = ggplot2.theme_gray
    @classmethod
    def new(cls, base_size = 12):
       res = cls(cls._constructor(base_size = base_size))
       return res
theme_gray = ThemeGray.new
theme_grey = theme_gray

class ThemeMinimal(Theme):
    _constructor = ggplot2.theme_minimal
    @classmethod
    def new(cls, base_size = 12, base_family = ""):
       res = cls(cls._constructor(base_size = base_size,
                                  base_family = base_family))
       return res

theme_minimal = ThemeMinimal.new

class ThemeLinedraw(Theme):
    _constructor = ggplot2.theme_linedraw
    @classmethod
    def new(cls, base_size=12, base_family=""):
       res = cls(cls._constructor(base_size=base_size,
                                  base_family=base_family))
       return res
theme_linedraw = ThemeLinedraw.new

class ThemeVoid(Theme):
    _constructor = ggplot2.theme_void
    @classmethod
    def new(cls, base_size = 12, base_family = ""):
       res = cls(cls._constructor(base_size = base_size,
                                  base_family = base_family))
       return res

theme_void = ThemeVoid.new


#theme_render
theme_set = ggplot2.theme_set

theme_update = ggplot2.theme_update  

gplot = ggplot2.qplot

map_data = ggplot2.map_data

theme = ggplot2_env['theme']

ggtitle = ggplot2.ggtitle

original_ri2ro = conversion.ri2ro
def ggplot2_conversion(robj):

    pyobj = original_ri2ro(robj)

    try:
       rcls = pyobj.rclass
    except AttributeError:
       # conversion lead to something that is no
       # longer an R object
       return pyobj

    if (rcls is not None) and ('gg' in rcls):
       pyobj = GGPlot(pyobj)

    return pyobj

conversion.ri2ro = ggplot2_conversion




import unittest
import rpy2.robjects as robjects
rinterface = robjects.rinterface
import rpy2.rlike.container as rlc

import array
import csv, tempfile

class DataFrameTestCase(unittest.TestCase):

    def testNewFromTaggedList(self):
        letters = robjects.r.letters        
        numbers = robjects.r('1:26')
        df = robjects.DataFrame(rlc.TaggedList((letters, numbers),
                                               tags = ('letters', 'numbers')))

        self.assertEqual("data.frame", df.rclass[0])

    def testNewFromRObject(self):
        numbers = robjects.r('1:5')
        self.assertRaises(ValueError, robjects.DataFrame, numbers)

        rfunc = robjects.r('sum')
        self.assertRaises(ValueError, robjects.DataFrame, rfunc)

        rdataf = robjects.r('data.frame(a=1:2, b=c("a", "b"))')
        dataf = robjects.DataFrame(rdataf)        

    def testNewFromOrdDict(self):
        od = rlc.OrdDict(c=(('a', robjects.IntVector((1,2))),
                            ('b', robjects.StrVector(('c', 'd')))
                            ))
        dataf = robjects.DataFrame(od)
        self.assertEqual(1, dataf.rx2('a')[0])
        
    def testDim(self):
        letters = robjects.r.letters        
        numbers = robjects.r('1:26')
        df = robjects.DataFrame(rlc.TaggedList((letters, numbers),
                                               tags = ('letters', 'numbers')))
        self.assertEqual(26, df.nrow)
        self.assertEqual(2, df.ncol)

    def testFrom_csvfile(self):
        column_names = ('letter', 'value')
        data = (column_names,
                ('a', 1),
                ('b', 2),
                ('c', 3))
        fh = tempfile.NamedTemporaryFile(mode = "w", delete = False)
        csv_w = csv.writer(fh)
        csv_w.writerows(data)
        fh.close()
        dataf = robjects.DataFrame.from_csvfile(fh.name)
        self.assertEqual(column_names, tuple(dataf.names))
        self.assertEqual(3, dataf.nrow)
        self.assertEqual(2, dataf.ncol)

    def testTo_csvfile(self):
        fh = tempfile.NamedTemporaryFile(mode = "w", delete = False)
        fh.close()
        d = {'letter': robjects.StrVector('abc'),
             'value' : robjects.IntVector((1, 2, 3))}
        dataf = robjects.DataFrame(d)
        dataf.to_csvfile(fh.name)
        dataf = robjects.DataFrame.from_csvfile(fh.name)
        self.assertEqual(3, dataf.nrow)
        self.assertEqual(2, dataf.ncol)

    def testIter_col(self):
        dataf = robjects.r('data.frame(a=1:2, b=I(c("a", "b")))')
        col_types = [x.typeof for x in dataf.iter_column()]
        self.assertEqual(rinterface.INTSXP, col_types[0])
        self.assertEqual(rinterface.STRSXP, col_types[1])

    def testIter_row(self):
        dataf = robjects.r('data.frame(a=1:2, b=I(c("a", "b")))')
        rows = [x for x in dataf.iter_row()]
        self.assertEqual(1, rows[0][0][0])
        self.assertEqual("b", rows[1][1][0])

    def testColnames(self):
        dataf = robjects.r('data.frame(a=1:2, b=I(c("a", "b")))')
        self.assertEqual('1', dataf.rownames[0])
        self.assertEqual('2', dataf.rownames[1])

    def testColnames_set(self):
        dataf = robjects.r('data.frame(a=1:2, b=I(c("a", "b")))')
        dataf.colnames = robjects.StrVector('de')
        self.assertEqual('d', dataf.colnames[0])
        self.assertEqual('e', dataf.colnames[1])

    def testRownames(self):
        dataf = robjects.r('data.frame(a=1:2, b=I(c("a", "b")))')
        self.assertEqual('a', dataf.colnames[0])
        self.assertEqual('b', dataf.colnames[1])        

    def testRownames_set(self):
        dataf = robjects.r('data.frame(a=1:2, b=I(c("a", "b")))')
        dataf.rownames = robjects.StrVector('de')
        self.assertEqual('d', dataf.rownames[0])
        self.assertEqual('e', dataf.rownames[1])

    def testCbind(self):
        dataf = robjects.r('data.frame(a=1:2, b=I(c("a", "b")))')
        dataf = dataf.cbind(robjects.r('data.frame(a=1:2, b=I(c("a", "b")))'))
        self.assertEqual(4, dataf.ncol)
        self.assertEqual(2, len([x for x in dataf.colnames if x == 'a']))

    def testCbind(self):
        dataf = robjects.r('data.frame(a=1:2, b=I(c("a", "b")))')
        dataf = dataf.cbind(a = robjects.StrVector(("c", "d")))
        self.assertEqual(3, dataf.ncol)
        self.assertEqual(2, len([x for x in dataf.colnames if x == 'a']))

def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(DataFrameTestCase)
    return suite

if __name__ == '__main__':
     unittest.main()



import rpy2.robjects as ro
import rpy2.robjects.conversion as conversion
import rpy2.rinterface as rinterface
from rpy2.rinterface import (Sexp,
                             SexpVector,
                             ListSexpVector,
                             StrSexpVector, ByteSexpVector,
                             LGLSXP, INTSXP, REALSXP, CPLXSXP,
                             STRSXP, VECSXP, NULL)
import numpy

#from rpy2.robjects.vectors import DataFrame, Vector, ListVector

original_converter = None

# The possible kind codes are listed at
#   http://numpy.scipy.org/array_interface.shtml
_kinds = {
    # "t" -> not really supported by numpy
    "b": rinterface.LGLSXP,
    "i": rinterface.INTSXP,
    # "u" -> special-cased below
    "f": rinterface.REALSXP,
    "c": rinterface.CPLXSXP,
    # "O" -> special-cased below
    "S": rinterface.STRSXP,
    "U": rinterface.STRSXP,
    # "V" -> special-cased below
    #FIXME: datetime64 ?
    #"datetime64":
    }

#FIXME: the following would need further thinking & testing on
#       32bits architectures
_kinds['float64'] = rinterface.REALSXP

_vectortypes = (rinterface.LGLSXP,
                rinterface.INTSXP,
                rinterface.REALSXP,
                rinterface.CPLXSXP,
                rinterface.STRSXP)

converter = conversion.Converter('original numpy conversion')
py2ri = converter.py2ri
py2ro = converter.py2ro
ri2py = converter.ri2py
ri2ro = converter.ri2ro

import sys

if sys.version_info[0] == 3:
    def numpy_O_py2ri(o):
        if all(isinstance(x, str) for x in o):
            res = StrSexpVector(o)
        elif all(isinstance(x, bytes) for x in o):
            res = ByteSexpVector(o)
        else:
            res = conversion.py2ri(list(o))
        return res
else:
    def numpy_O_py2ri(o):
        if all((isinstance(x, str) or isinstance(x, bytes) or isinstance(x, unicode)) for x in o):
            res = StrSexpVector(o)
        else:
            res = conversion.py2ri(list(o))
        return res
    
@py2ri.register(numpy.ndarray)
def numpy2ri(o):
    """ Augmented conversion function, converting numpy arrays into
    rpy2.rinterface-level R structures. """
    if not o.dtype.isnative:
        raise(ValueError("Cannot pass numpy arrays with non-native byte orders at the moment."))

    # Most types map onto R arrays:
    if o.dtype.kind in _kinds:
        # "F" means "use column-major order"
        vec = SexpVector(o.ravel("F"), _kinds[o.dtype.kind])
        dim = SexpVector(o.shape, INTSXP)
        #FIXME: no dimnames ?
        #FIXME: optimize what is below needed/possible ? (other ways to create R arrays ?)
        res = rinterface.baseenv['array'](vec, dim=dim)
    # R does not support unsigned types:
    elif o.dtype.kind == "u":
        raise(ValueError("Cannot convert numpy array of unsigned values -- R does not have unsigned integers."))
    # Array-of-PyObject is treated like a Python list:
    elif o.dtype.kind == "O":
        res = numpy_O_py2ri(o)
    # Record arrays map onto R data frames:
    elif o.dtype.kind == "V":
        if o.dtype.names is None:
            raise(ValueError("Nothing can be done for this numpy array type %s at the moment." % (o.dtype,)))
        df_args = []
        for field_name in o.dtype.names:
            df_args.append((field_name,
                            conversion.py2ri(o[field_name])))
        res = ro.baseenv["data.frame"].rcall(tuple(df_args), ro.globalenv)
    # It should be impossible to get here:
    else:
        raise(ValueError("Unknown numpy array type '%s'." % str(o.dtype)))
    return res

@py2ri.register(numpy.integer)
def npint_py2ri(obj):
    return ro.int2ri(obj)

@py2ri.register(numpy.floating)
def npfloat_py2ri(obj):
    return rinterface.SexpVector([obj, ], rinterface.REALSXP)

@py2ri.register(object)
def nonnumpy2ri(obj):
    # allow array-likes to also function with this module.
    if not isinstance(obj, numpy.ndarray) and hasattr(obj, '__array__'):
        obj = obj.__array__()
        return ro.default_converter.py2ri(obj)
    else:
        raise original_converter.py2ri(obj)

@py2ro.register(numpy.ndarray)
def numpy2ro(obj):
    res = numpy2ri(obj)
    return ro.vectors.rtypeof2rotype[res.typeof](res)

@ri2py.register(ListSexpVector)
def ri2py_list(obj):
    if 'data.frame' in obj.rclass:
        # R "factor" vectors will not convert well by default
        # (will become integers), so we build a temporary list o2
        # with the factors as strings.
        o2 = list()
        # An added complication is that the conversion defined
        # in this module will make __getitem__ at the robjects
        # level return numpy arrays
        for column in rinterface.ListSexpVector(obj):
            if 'factor' in column.rclass:
                levels = tuple(column.do_slot("levels"))
                column = tuple(levels[x-1] for x in column)
            o2.append(column)
        names = obj.do_slot('names')
        if names is NULL:
            res = numpy.rec.fromarrays(o2)
        else:
            res = numpy.rec.fromarrays(o2, names=tuple(names))
    else:
        # not a data.frame, yet is it still possible to convert it
        res = ro.default_converter.ri2py(obj)
    return res

@ri2py.register(Sexp)
def ri2py_sexp(obj):
    if (obj.typeof in _vectortypes) and (obj.typeof != VECSXP):
        res = numpy.asarray(obj)
    else:
        res = ro.default_converter.ri2py(obj)
    return res

def activate():
    global original_converter

    # If module is already activated, there is nothing to do
    if original_converter is not None:
        return

    original_converter = conversion.converter
    new_converter = conversion.Converter('numpy conversion',
                                         template=original_converter)

    for k,v in py2ri.registry.items():
        if k is object:
            continue
        new_converter.py2ri.register(k, v)

    for k,v in ri2ro.registry.items():
        if k is object:
            continue
        new_converter.ri2ro.register(k, v)

    for k,v in py2ro.registry.items():
        if k is object:
            continue
        new_converter.py2ro.register(k, v)

    for k,v in ri2py.registry.items():
        if k is object:
            continue
        new_converter.ri2py.register(k, v)

    conversion.set_conversion(new_converter)


def deactivate():
    global original_converter
    # If module has never been activated or already deactivated,
    # there is nothing to do
    if original_converter is None:
        return

    conversion.set_conversion(original_converter)
    original_converter = None



"""
R objects as Python objects.

The module is structured around the singleton r of class R,
that represents an embedded R.

License: GPLv2+

"""

import os, sys
import types
import array
import itertools
from datetime import datetime
import rpy2.rinterface as rinterface
import rpy2.rlike.container as rlc

from rpy2.robjects.robject import RObjectMixin, RObject
from rpy2.robjects.vectors import (BoolVector,
                                   IntVector,
                                   FloatVector,
                                   ComplexVector,
                                   StrVector,
                                   FactorVector,
                                   Vector,
                                   ListVector,
                                   DateVector,
                                   POSIXct,
                                   POSIXlt,
                                   Array,
                                   Matrix,
                                   DataFrame)
from rpy2.robjects.functions import Function, SignatureTranslatedFunction
from rpy2.robjects.environments import Environment
from rpy2.robjects.methods import RS4

from . import conversion

from rpy2.rinterface import (Sexp, 
                             SexpVector, 
                             SexpClosure, 
                             SexpEnvironment, 
                             SexpS4,
                             StrSexpVector,
                             SexpExtPtr)
_globalenv = rinterface.globalenv

# missing values
from rpy2.rinterface import (NA_Real, 
                             NA_Integer, 
                             NA_Logical, 
                             NA_Character, 
                             NA_Complex, 
                             NULL)

if sys.version_info[0] == 2:
    py3str = unicode
    py3bytes = str
    import itertools
    zip = itertools.izip
else:
    long = int
    py3str = str
    py3bytes = bytes

_rparse = rinterface.baseenv['parse']
_reval = rinterface.baseenv['eval']

def reval(string, envir = _globalenv):
    """ Evaluate a string as R code
    - string: a string
    - envir: an environment in which the environment should take place
             (default: R's global environment)
    """
    p = rinterface.parse(string)
    res = _reval(p, envir = envir)
    return res

default_converter = conversion.Converter('base empty converter')

@default_converter.ri2ro.register(RObject)
def _(obj):
    return obj


def sexpvector_to_ro(obj):
    try:
        rcls = obj.do_slot("class")
    except LookupError as le:
        rcls = [None]

    if 'data.frame' in rcls:
        res = vectors.DataFrame(obj)
        return res
    try:
        dim = obj.do_slot("dim")
        if len(dim) == 2:
            res = vectors.Matrix(obj)
        else:
            res = vectors.Array(obj)
    except LookupError as le:
        if obj.typeof == rinterface.INTSXP:
            if 'factor' in rcls:
                res = vectors.FactorVector(obj)
            else:
                res = vectors.IntVector(obj)
        elif obj.typeof == rinterface.REALSXP:
            if obj.rclass[0] == 'POSIXct':
                res = vectors.POSIXct(obj)
            else:
                res = vectors.FloatVector(obj)
        elif obj.typeof == rinterface.LGLSXP:
            res = vectors.BoolVector(obj)
        elif obj.typeof == rinterface.STRSXP:
            res = vectors.StrVector(obj)
        elif obj.typeof == rinterface.VECSXP:
            res = vectors.ListVector(obj)
        elif obj.typeof == rinterface.LANGSXP and 'formula' in rcls:
            res = Formula(obj)
        else:
            res = vectors.Vector(obj)
    return res

default_converter.ri2ro.register(SexpVector, sexpvector_to_ro)

TYPEORDER = {bool: (0, BoolVector),
             int: (1, IntVector),
             float: (2, FloatVector),
             complex: (3, ComplexVector),
             str: (4, StrVector)}
def sequence_to_vector(lst):
    curr_typeorder = -1
    i = None
    for i, elt in enumerate(lst):
        cls = type(elt)
        if cls in TYPEORDER:
            if TYPEORDER[cls][0] > curr_typeorder:
                curr_typeorder, curr_type = TYPEORDER[cls]
        else:
            raise ValueError('The element %i in the list has a type that cannot be handled.' % i)
    if i is None:
        raise ValueError('The parameter "lst" is an empty sequence. The type of the corresponding R vector cannot be determined.')
    res = curr_type(lst)
    return res


@default_converter.ri2ro.register(SexpClosure)
def _(obj):
    return SignatureTranslatedFunction(obj)

@default_converter.ri2ro.register(SexpEnvironment)
def _(obj):
    return Environment(obj)

@default_converter.ri2ro.register(SexpS4)
def _(obj):
    return RS4(obj)

@default_converter.ri2ro.register(SexpExtPtr)
def _(obj):
    return obj

@default_converter.ri2ro.register(object)
def _(obj):
    return RObject(obj)

@default_converter.ri2ro.register(type(NULL))
def _(obj):
    return obj


def default_py2ri(o):
    """ Convert an arbitrary Python object to a
    :class:`rpy2.rinterface.Sexp` object.
    Creates an R object with the content of the Python object,
    wich means data copying.
    :param o: object
    :rtype: :class:`rpy2.rinterface.Sexp` (and subclasses)
    """
    pass

@default_converter.py2ri.register(RObject)
def _(obj):
    return rinterface.Sexp(obj)

@default_converter.py2ri.register(Sexp)
def _(obj):
    return obj

@default_converter.py2ri.register(array.array)
def _(obj):
    if obj.typecode in ('h', 'H', 'i', 'I'):
        res = rinterface.SexpVector(obj, rinterface.INTSXP)
    elif obj.typecode in ('f', 'd'):
        res = rinterface.SexpVector(obj, rinterface.REALSXP)
    else:
        raise(ValueError("Nothing can be done for this array type at the moment."))
    return res

@default_converter.py2ri.register(bool)
def _(obj):
    return rinterface.SexpVector([obj, ], rinterface.LGLSXP)

def int2ri(obj):
    # special case for NA_Logical
    if obj is rinterface.NA_Logical:
        res = rinterface.SexpVector([obj, ], rinterface.LGLSXP)
    else:
        res = rinterface.SexpVector([obj, ], rinterface.INTSXP)
    return res

default_converter.py2ri.register(int, int2ri)
default_converter.py2ri.register(long, int2ri)

@default_converter.py2ri.register(float)
def _(obj):
    return rinterface.SexpVector([obj, ], rinterface.REALSXP)

@default_converter.py2ri.register(py3bytes)
def _(obj):
    return rinterface.SexpVector([obj, ], rinterface.STRSXP)

@default_converter.py2ri.register(py3str)
def _(obj):
    return rinterface.SexpVector([obj, ], rinterface.STRSXP)

@default_converter.py2ri.register(list)
def _(obj):
    return rinterface.ListSexpVector([conversion.py2ri(x) for x in obj])

@default_converter.py2ri.register(rlc.TaggedList)
def _(obj):
    res = rinterface.ListSexpVector([conversion.py2ri(x) for x in obj])
    res.do_slot_assign('names', rinterface.StrSexpVector(obj.tags))
    return res

@default_converter.py2ri.register(complex)
def _(obj):
    return rinterface.SexpVector([obj, ], rinterface.CPLXSXP)


@default_converter.py2ro.register(object)
def _(obj):
    robj = conversion.py2ri(obj)
    return conversion.ri2ro(robj)

@default_converter.py2ri.register(types.FunctionType)
def _function_to_ri(func):
    def wrap(*args):
        res = func(*args)
        res = conversion.py2ro(res)
        return res
    rfunc = rinterface.rternalize(wrap)
    return rfunc

@default_converter.ri2py.register(object)
def _(obj):
    return conversion.ri2ro(obj)

class Formula(RObjectMixin, rinterface.Sexp):

    def __init__(self, formula, environment = _globalenv):
        if isinstance(formula, str):
            inpackage = rinterface.baseenv["::"]
            asformula = inpackage(rinterface.StrSexpVector(['stats', ]),
                                  rinterface.StrSexpVector(['as.formula', ]))
            formula = rinterface.SexpVector(rinterface.StrSexpVector([formula, ]))
            robj = asformula(formula,
                             env = environment)
        else:
            robj = formula
        super(Formula, self).__init__(robj)

    def getenvironment(self):
        """ Get the environment in which the formula is finding its symbols."""
        res = self.do_slot(".Environment")
        res = conversion.ri2ro(res)
        return res

    def setenvironment(self, val):
        """ Set the environment in which a formula will find its symbols."""
        if not isinstance(val, rinterface.SexpEnvironment):
            raise ValueError("The environment must be an instance of" +
                             " rpy2.rinterface.Sexp.environment")
        self.do_slot_assign(".Environment", val)

    environment = property(getenvironment, setenvironment,
                           "R environment in which the formula will look for" +
                           " its variables.")

    
class ParsedCode(rinterface.SexpVector):
    def __init__(self, source=None):
        self._source = source
        super(ParsedCode, self).__init__(self)

    @property
    def source(self):
        return self._source

    
class SourceCode(str):
    
    def parse(self, keep_source=True):
        res = _rparse(text=rinterface.StrSexpVector((self,)))
        if keep_source:
            res = ParsedCode(res, source=self)
        else:
            res = ParsedCode(res, source=None)
        return res
    
    def as_namespace(self, name):
        """ Name for the namespace """
        return SignatureTranslatedAnonymousPackage(self,
                                                   name)

class R(object):
    """
    Singleton representing the embedded R running.
    """
    _instance = None

    def __new__(cls):
        if cls._instance is None:
            rinterface.initr()
            cls._instance = object.__new__(cls)
        return cls._instance

    def __getattribute__(self, attr):
        try:
            return super(R, self).__getattribute__(attr)
        except AttributeError as ae:
            orig_ae = str(ae)

        try:
            return self.__getitem__(attr)
        except LookupError as le:
            raise AttributeError(orig_ae)

    def __getitem__(self, item):
        res = _globalenv.get(item)
        res = conversion.ri2py(res)
        if hasattr(res, '__rname__'):
            res.__rname__ = item
        return res

    #FIXME: check that this is properly working
    def __cleanup__(self):
        rinterface.endEmbeddedR()
        del(self)

    def __str__(self):
        s = super(R, self).__str__()
        s += os.linesep
        version = self["version"]
        tmp = [n+': '+val[0] for n, val in zip(version.names, version)]
        s += str.join(os.linesep, tmp)
        return s

    def __call__(self, string):
        p = _rparse(text=StrSexpVector((string,)))
        res = self.eval(p)
        return conversion.ri2py(res)

r = R()

conversion.set_conversion(default_converter)

globalenv = conversion.converter.ri2ro(_globalenv)
baseenv = conversion.converter.ri2ro(rinterface.baseenv)
emptyenv = conversion.converter.ri2ro(rinterface.emptyenv)




#!/usr/bin/env python

'''tests.py - run all the tests worth running

The goal is that "ERRORS" and "FAILURES" are true failures, and expected
problems sould be dealt with using decorators.'''
from __future__ import print_function

# if `singledispatch` is absent, the unit tests are failing with a rather
# obscure / misleading error message. Test it now and report the problem. 
import sys
if sys.version_info[0] < 3 or (sys.version_info[0] == 3 and sys.version_info[1] < 4):
    try:
        from singledispatch import singledispatch
    except ImportError as ie:
        print('The module "singledispatch is required for Python < 3.4')
        raise ie

from os.path import dirname
import unittest

import rpy2
import rpy2.tests_rpy_classic

def load_tests(loader, tests, pattern):
    '''Run tests a little faster than with TestLoader.discover()

    Note that we are using the unittest API here, but blithely ignore the values
    passed in for `tests` and `pattern`'''
    # For some reason, the commented code directly below is slow and loads some
    # things twice One specific message is regarding package_dependencies from
    # the tools package.
    # rpy_root = dirname(rpy2.__file__)
    # alltests = unittest.defaultTestLoader.discover(rpy_root, pattern='test*')

    rpy_root = dirname(rpy2.__file__)
    tests = unittest.TestSuite()
    pattern = 'test*'

    # This now catches some extra tests (bypassing the suite() functions),
    # at least in a virtualenv that lacks various packages, like numpy &
    # pandas
    suite_robjects = loader.discover('robjects', pattern, rpy_root)
    suite_rinterface = loader.discover('rinterface', pattern, rpy_root)
    suite_rlike = loader.discover('rlike', pattern, rpy_root)
    # This is once again testless
    suite_interactive = loader.discover('interactive', pattern, rpy_root)
    # ipython is a little trickier because it is an optional
    # dependency.
    try:
        import IPython
        has_ipython = True
    except ImportError as ie:
        has_ipython = False

    if has_ipython:
        suite_ipython= loader.discover('ipython', pattern, rpy_root)
    else:
        class MissingIpythonTestCase(unittest.TestCase):
            @unittest.skip("The optional dependency IPython is required in order to test features using it.")
            def testHasIpython(self):
                pass
        suite_ipython = unittest.TestLoader().loadTestsFromTestCase(MissingIpythonTestCase)

    suite_rpy_classic = rpy2.tests_rpy_classic.suite()

    tests.addTests([suite_rinterface,
                    suite_robjects,
                    suite_rlike,
                    suite_interactive,
                    suite_ipython,
                    suite_rpy_classic
                    ])
    return tests


if __name__ == "__main__":
    import sys, rpy2.rinterface
    print("rpy2 version: %s" % rpy2.__version__)
    print("- built against R version: %s" % '-'.join(str(x)
          for x in rpy2.rinterface.R_VERSION_BUILD))

    try:
        import rpy2.rinterface
    except Exception as e:
        print("'rpy2.rinterface' could not be imported:")
        print(e)
        sys.exit(1)
    try:
        rpy2.rinterface.initr()
    except Exception as e:
        print("- The embedded R could not be initialized")
        print(e)
        sys.exit(1)
    try:
        rv = rpy2.rinterface.baseenv['R.version.string']
        print("- running linked to R version: %s" % rv[0])
    except KeyError as ke:
        print("The R version dynamically linked cannot be identified.")

    # This will sys.exit() with an appropriate error code
    unittest.main(buffer=True)



import rpy2.robjects as ro
import rpy2.robjects.conversion as conversion
import rpy2.rinterface as rinterface
from rpy2.rinterface import (SexpVector,
                             StrSexpVector,
                             IntSexpVector,
                             INTSXP)

from pandas.core.frame import DataFrame as PandasDataFrame
from pandas.core.series import Series as PandasSeries
from pandas.core.index import Index as PandasIndex
import pandas
from numpy import recarray
import numpy
import warnings

from collections import OrderedDict
from rpy2.robjects.vectors import (DataFrame,
                                   Vector,
                                   FactorVector,
                                   ListVector,
                                   StrVector,
                                   IntVector,
                                   POSIXct)
from rpy2.rinterface import (IntSexpVector,
                             ListSexpVector)
original_converter = None 

# pandas is requiring numpy. We add the numpy conversion will be
# activate in the function activate() below
import rpy2.robjects.numpy2ri as numpy2ri


ISOdatetime = rinterface.baseenv['ISOdatetime']
as_vector = rinterface.baseenv['as.vector']

converter = conversion.Converter('original pandas conversion')
py2ri = converter.py2ri
py2ro = converter.py2ro
ri2py = converter.ri2py
ri2ro = converter.ri2ro

# numpy types for Pandas columns that require (even more) special handling
dt_datetime64ns_type = numpy.dtype('datetime64[ns]')
dt_O_type = numpy.dtype('O')

@py2ri.register(PandasDataFrame)
def py2ri_pandasdataframe(obj):
    od = OrderedDict()
    for name, values in obj.iteritems():
        try:
            od[name] = conversion.py2ri(values)
        except Exception as e:
            warnings.warn('Error while trying to convert '
                          'the column "%s". Fall back to string conversion. '
                          'The error is: %s' %\
                          (name, str(e)))
            od[name] = StrVector(values)
        
    return DataFrame(od)

@py2ri.register(PandasIndex)
def py2ri_pandasindex(obj):
    if obj.dtype.kind == 'O':
        return StrVector(obj)
    else:
        # pandas2ri should definitely not have to know which paths remain to be
        # converted by numpy2ri
        # Answer: the thing is that pandas2ri builds on the conversion
        # rules defined by numpy2ri - deferring to numpy2ri is allowing
        # us to reuse that code.
        return numpy2ri.numpy2ri(obj)


def py2ri_categoryseries(obj):
    for c in obj.cat.categories:
        if not isinstance(c, str):
            raise ValueError('Converting pandas "Category" series to R factor is only possible when categories are strings.')
    res = IntSexpVector(list(x+1 for x in obj.cat.codes))
    res.do_slot_assign('levels', StrSexpVector(obj.cat.categories))
    if obj.cat.ordered:
        res.rclass = StrSexpVector('ordered', 'factor')
    else:
        res.rclass = StrSexpVector('factor')
    return res

@py2ri.register(PandasSeries)
def py2ri_pandasseries(obj):
    if obj.dtype.name == 'category':
        res = py2ri_categoryseries(obj)
        res = FactorVector(res)
    elif obj.dtype == dt_datetime64ns_type:
        # time series
        d = [IntVector([x.year for x in obj]),
             IntVector([x.month for x in obj]),
             IntVector([x.day for x in obj]),
             IntVector([x.hour for x in obj]),
             IntVector([x.minute for x in obj]),
             IntVector([x.second for x in obj])]
        res = ISOdatetime(*d)
        #FIXME: can the POSIXct be created from the POSIXct constructor ?
        # (is '<M8[ns]' mapping to Python datetime.datetime ?)
        res = POSIXct(res)
    else:
        # converted as a numpy array
        func = numpy2ri.converter.py2ri.registry[numpy.ndarray]
        # current conversion as performed by numpy
        res = func(obj)
        if len(obj.shape) == 1:
            if (obj.dtype != dt_O_type):
                # force into an R vector
                res=as_vector(res)

    # "index" is equivalent to "names" in R
    if obj.ndim == 1:
        res.do_slot_assign('names',
                           StrVector(tuple(str(x) for x in obj.index)))
    else:
        res.do_slot_assign('dimnames',
                           SexpVector(conversion.py2ri(obj.index)))
    return res

@ri2py.register(SexpVector)
def ri2py_vector(obj):
    res = numpy2ri.ri2py(obj)
    return res
    
@ri2py.register(IntSexpVector)
def ri2py_intvector(obj):
    # special case for factors
    if 'factor' in obj.rclass:
        res = pandas.Categorical.from_codes(numpy.asarray(obj) - 1,
                                            categories = obj.do_slot('levels'),
                                            ordered = 'ordered' in obj.rclass)
    else:
        res = numpy2ri.ri2py(obj)
    return res

@ri2py.register(ListSexpVector)
def ri2py_listvector(obj):        
    if 'data.frame' in obj.rclass:
        res = ri2py.registry[DataFrame](obj)
    else:
        res = numpy2ri.ri2py(obj)
    return res

@ri2py.register(DataFrame)
def ri2py_dataframe(obj):
    # use the numpy converter
    recarray = numpy2ri.ri2py(obj)
    try:
        idx = numpy2ri.ri2py(obj.do_slot('row.names'))
    except LookupError as le:
        idx = None
    res = PandasDataFrame.from_records(recarray,
                                       index=idx)
    return res

def activate():
    global original_converter
    # If module is already activated, there is nothing to do
    if original_converter is not None: 
        return

    original_converter = conversion.Converter('snapshot before pandas conversion',
                                              template=conversion.converter)
    numpy2ri.activate()
    new_converter = conversion.Converter('snapshot before pandas conversion',
                                         template=conversion.converter)
    numpy2ri.deactivate()

    for k,v in py2ri.registry.items():
        if k is object:
            continue
        new_converter.py2ri.register(k, v)

    for k,v in ri2ro.registry.items():
        if k is object:
            continue
        new_converter.ri2ro.register(k, v)
    
    for k,v in py2ro.registry.items():
        if k is object:
            continue
        new_converter.py2ro.register(k, v)

    for k,v in ri2py.registry.items():
        if k is object:
            continue
        new_converter.ri2py.register(k, v)

    conversion.set_conversion(new_converter)

def deactivate():
    global original_converter

    # If module has never been activated or already deactivated,
    # there is nothing to do
    if original_converter is None:
        return

    conversion.set_conversion(original_converter)
    original_converter = None




import unittest
import rpy2.robjects as robjects
import rpy2.robjects.language as lg
rinterface = robjects.rinterface


class LanguageTestCase(unittest.TestCase):

    def setUp(self):
        for var in ('x', 'y'):
            if var in robjects.globalenv.keys():
                #del(ro.globalenv[var]) #segfault
                pass

    def tearDown(self):
        for var in ('x', 'y'):
            if var in robjects.globalenv.keys():
                #del(ro.globalenv[var]) #segfault
                pass

    def testEval(self):
        code = """
x <- 1+2
y <- (x+1) / 2
"""
        res = lg.eval(code)
        self.assertTrue('x' in robjects.globalenv.keys())
        self.assertEqual(3, robjects.globalenv['x'][0])
        self.assertTrue('y' in robjects.globalenv.keys())
        self.assertEqual(2, robjects.globalenv['y'][0])

    def testEvalInEnvironment(self):
        code = """
x <- 1+2
y <- (x+1) / 2
"""
        env = robjects.Environment()
        res = lg.eval(code, envir=env)
        self.assertTrue('x' in env.keys())
        self.assertEqual(3, env['x'][0])
        self.assertTrue('y' in env.keys())
        self.assertEqual(2, env['y'][0])

def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(LanguageTestCase)
    return suite

if __name__ == '__main__':
     unittest.main()



import unittest
import itertools
import rpy2.rinterface as rinterface


try:
    import numpy
    has_numpy = True
except ImportError:
    has_numpy = False

def only_numpy(function):
    def res(self):
        if has_numpy:
            return function(self)
        else:
            return None

rinterface.initr()

def floatEqual(x, y, epsilon = 0.00000001):
    return abs(x - y) < epsilon

def testArrayStructInt(self, numericModule):
    px = [1, -2, 3]
    x = rinterface.SexpVector(px, rinterface.INTSXP)
    nx = numericModule.asarray(x)
    self.assertEqual(nx.dtype.kind, 'i')
    for orig, new in zip(px, nx):
        self.assertEqual(orig, new)

    # change value in the Python array... makes it change in the R vector
    nx[1] = 12
    self.assertEqual(x[1], 12)

def testArrayStructDouble(self, numericModule):
    px = [1.0, -2.0, 3.0]
    x = rinterface.SexpVector(px, rinterface.REALSXP)
    nx = numericModule.asarray(x)
    self.assertEqual(nx.dtype.kind, 'f')
    for orig, new in itertools.izip(px, nx):
        self.assertEqual(orig, new)
    
    # change value in the Python array... makes it change in the R vector
    nx[1] = 333.2
    self.assertEqual(x[1], 333.2)

def testArrayStructComplex(self, numericModule):
    px = [1+2j, 2+5j, -1+0j]
    x = rinterface.SexpVector(px, rinterface.CPLXSXP)
    nx = numericModule.asarray(x)
    self.assertEqual(nx.dtype.kind, 'c')
    for orig, new in itertools.izip(px, nx):
        self.assertEqual(orig, new)
    
def testArrayStructBoolean(self, numericModule):
    px = [True, False, True]
    x = rinterface.SexpVector(px, rinterface.LGLSXP)
    nx = numericModule.asarray(x)
    self.assertEqual('i', nx.dtype.kind) # not 'b', see comments in array.c
    for orig, new in itertools.izip(px, nx):
        self.assertEqual(orig, new)


class SexpVectorNumericTestCase(unittest.TestCase):

    @only_numpy
    def testArrayStructNumpyInt(self):
        testArrayStructInt(self, numpy)
    @only_numpy
    def testArrayStructNumpyDouble(self):
        testArrayStructDouble(self, numpy)
    @only_numpy
    def testArrayStructNumpyComplex(self):
        testArrayStructComplex(self, numpy)
    @only_numpy
    def testArrayStructNumpyBoolean(self):
        testArrayStructBoolean(self, numpy)
    @only_numpy
    def testArrayShapeLen3(self):
        extract = rinterface.baseenv['[']
        rarray = rinterface.baseenv['array'](rinterface.IntSexpVector(range(30)),
                                             dim = rinterface.IntSexpVector([5,2,3]))
        npyarray = numpy.array(rarray)
        for i in range(5):
            for j in range(2):
                for k in range(3):
                    self.assertEqual(extract(rarray, i+1, j+1, k+1)[0], 
                                      npyarray[i, j, k])

def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(SexpVectorNumericTestCase)
    return suite

if __name__ == '__main__':
    tr = unittest.TextTestRunner(verbosity = 2)
    tr.run(suite())




"""
The module contains the conversion functions to be
used by the rpy2.robjects functions and methods.

These functions are initially empty place-holders,
raising a NotImplementedError exception.
"""

import sys
from collections import namedtuple

if sys.version_info[0] < 3 or (sys.version_info[0] == 3 and sys.version_info[1] < 4):
    from singledispatch import singledispatch
else:
    from functools import singledispatch

def overlay_converter(src, target):
    """
    :param src: source of additional conversion rules
    :type src: :class:`Converter`
    :param target: target. The conversion rules in the src will
                   be added to this object.
    :type target: :class:`Converter`
    """
    for k,v in src.ri2ro.registry.items():
        # skip the root dispatch
        if k is object and v is _ri2ro:
            continue
        target._ri2ro.register(k, v)
    for k,v in src.py2ri.registry.items():
        # skip the root dispatch
        if k is object and v is _py2ri:
            continue
        target._py2ri.register(k, v)
    for k,v in src.py2ro.registry.items():
        # skip the root dispatch
        if k is object and v is _py2ro:
            continue
        target._py2ro.register(k, v)
    for k,v in src.ri2py.registry.items():
        # skip the root dispatch
        if k is object and v is _ri2py:
            continue
        target._ri2py.register(k, v)

def _ri2ro(obj):
    """ Dummy function for ri2ro.

    This function will convert rpy2.rinterface (ri) low-level objects
    into rpy2.robjects (ro) higher-level objects.
    """
    raise NotImplementedError("Conversion 'ri2ro' not defined for objects of type '%s'" % str(type(obj)))

def _py2ri(obj):
    """ Dummy function for py2ri.
    
    This function will convert Python objects into rpy2.rinterface
    (ri) objects.
    """
    raise NotImplementedError("Conversion 'py2ri' not defined for objects of type '%s'" % str(type(obj)))

def _py2ro(obj):
    """ Dummy function for py2ro.

    This function will convert Python objects into rpy2.robjects
    (ro) objects.
    """
    raise NotImplementedError("Conversion 'py2ro' not defined for objects of type '%s'" % str(type(obj)))

def _ri2py(obj):
    """ Dummy function for ri2py.

    This function will convert Python objects into Python (presumably non-rpy2) objects.
    """
    raise NotImplementedError("Conversion 'ri2py' not defined for objects of type '%s'" % str(type(obj)))


class Converter(object):
    """
    Conversion between rpy2's low-level and high-level proxy objects
    for R objects, and Python (no R) objects.

    Converter objects can be added, the result being
    a Converter objects combining the translation rules from the
    different converters.
    """
    name = property(lambda self: self._name)
    ri2ro = property(lambda self: self._ri2ro)
    py2ri = property(lambda self: self._py2ri)
    py2ro = property(lambda self: self._py2ro)
    ri2py = property(lambda self: self._ri2py)
    lineage = property(lambda self: self._lineage)
    
    def __init__(self, name,
                 template=None):
        (ri2ro, py2ri, py2ro, ri2py) = Converter.make_dispatch_functions()
        self._name = name
        self._ri2ro = ri2ro
        self._py2ri = py2ri
        self._py2ro = py2ro
        self._ri2py = ri2py

        if template is None:
            lineage = tuple()
        else:
            lineage = list(template.lineage)
            lineage.append(name)
            lineage = tuple(lineage)
            overlay_converter(template, self)
        self._lineage = lineage

    def __add__(self, converter):
        assert isinstance(converter, Converter)
        new_name = '%s + %s' % (self.name, converter.name)
        # create a copy of `self` as the result converter
        result_converter = Converter(new_name, template=self)
        overlay_converter(converter, result_converter)
        return result_converter
    
    @staticmethod
    def make_dispatch_functions():
        ri2ro = singledispatch(_ri2ro)
        py2ri = singledispatch(_py2ri)
        py2ro = singledispatch(_py2ro)
        ri2py = singledispatch(_ri2py)

        return (ri2ro, py2ri, py2ro, ri2py)


class ConversionContext(object):
    """
    Context manager for instances of class Converter.
    """
    def __init__(self, ctx_converter):
        assert isinstance(ctx_converter, Converter)
        self._original_converter = converter
        self.ctx_converter = Converter('Converter-%i-in-context' % id(self),
                                       template=ctx_converter)

    def __enter__(self):
        set_conversion(self.ctx_converter)
        return self.ctx_converter

    def __exit__(self, exc_type, exc_val, exc_tb):
        set_conversion(self._original_converter)
        return False

localconverter = ConversionContext
    
converter = None
py2ri = None
py2ro = None
ri2ro = None
ri2py = None

def set_conversion(this_converter):
    """
    Set conversion rules in the conversion module.
    :param this_converter: The conversion rules
    :type this_converter: :class:`Converter`
    """
    global converter, py2ri, py2ro, ri2ro, ri2py
    converter = this_converter
    py2ri = converter.py2ri
    py2ro = converter.py2ro
    ri2ro = converter.ri2ro
    ri2py = converter.ri2py

set_conversion(Converter('base converter'))



import unittest
import rpy2.robjects as robjects
rinterface = robjects.rinterface

class FormulaTestCase(unittest.TestCase):

    def testNew(self):
        fml = robjects.Formula("y ~ x")
        self.assertEqual("formula", fml.rclass[0])
        
    def testGetenvironment(self):
        fml = robjects.Formula("y ~ x")
        env = fml.getenvironment()
        self.assertEqual("environment", env.rclass[0])

    def testSetenvironment(self):
        fml = robjects.Formula("y ~ x")
        newenv = robjects.baseenv['new.env']()
        env = fml.getenvironment()
        self.assertFalse(newenv.rsame(env))
        fml.setenvironment(newenv)
        env = fml.getenvironment()
        self.assertTrue(newenv.rsame(env))

def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(FormulaTestCase)
    return suite

if __name__ == '__main__':
     unittest.main()



from collections import namedtuple
from rpy2.robjects.packages import importr, data, WeakPackage
import warnings
with warnings.catch_warnings():
    warnings.simplefilter("ignore")
    tidyr = importr('tidyr', on_conflict="warn")
    
TARGET_VERSION = '0.4.1'
if tidyr.__version__ != TARGET_VERSION:
    warnings.warn('This was designed againt tidyr version %s but you have %s' % (TARGET_VERSION, tidyr.__version__))

tidyr = WeakPackage(tidyr._env,
                    tidyr.__rname__,
                    translation=tidyr._translation,
                    exported_names=tidyr._exported_names,
                    on_conflict="warn",
                    version=tidyr.__version__,
                    symbol_r2python=tidyr._symbol_r2python,
                    symbol_check_after=tidyr._symbol_check_after)


    
from rpy2.robjects.lib import dplyr

class DataFrame(dplyr.DataFrame):
    pass

DataFrame.summarize = dplyr._wrap(dplyr.summarize, DataFrame)
DataFrame.summarise = DataFrame.summarize

def _wrap(rfunc):
    def func(dataf, *args, **kwargs):
        cls = type(dataf)
        res = rfunc(dataf, *args, **kwargs)
        return cls(res)
    return func

DataFrame.gather = _wrap(tidyr.gather_)
DataFrame.spread = _wrap(tidyr.spread_)




import unittest
import sys
import rpy2.rinterface as rinterface

if sys.version_info[0] == 3:
    is_py3 = True
else:
    is_py3 = False

rinterface.initr()

class SexpEnvironmentTestCase(unittest.TestCase):

    def setUp(self):
        self.console = rinterface.get_writeconsole_regular()
        def noconsole(x):
            pass
        rinterface.set_writeconsole_regular(noconsole)

    def tearDown(self):
        rinterface.set_writeconsole_regular(self.console)

    def testNew(self):
        sexp = rinterface.globalenv
        sexp_new = rinterface.SexpEnvironment(sexp)

        idem = rinterface.globalenv.get("identical")
        self.assertTrue(idem(sexp, sexp_new)[0])

        sexp_new2 = rinterface.Sexp(sexp)
        self.assertTrue(idem(sexp, sexp_new2)[0])
        del(sexp)
        self.assertTrue(idem(sexp_new, sexp_new2)[0])

        self.assertRaises(ValueError, rinterface.SexpEnvironment, '2')

    def testGlobalEnv(self):
        ok = isinstance(rinterface.globalenv, rinterface.SexpEnvironment) 
        self.assertTrue(ok)

    def testGetClosure(self):
        help_R = rinterface.globalenv.get("help")
        ok = isinstance(help_R, rinterface.SexpClosure)
        self.assertTrue(ok)

    def testGetVector(self):
        pi_R = rinterface.globalenv.get("pi")
        ok = isinstance(pi_R, rinterface.SexpVector)
        self.assertTrue(ok)

    def testGetEnvironment(self):
        ge_R = rinterface.globalenv.get(".GlobalEnv")
        ok = isinstance(ge_R, rinterface.SexpEnvironment)
        self.assertTrue(ok)

    def testGetOnlyFromLoadedLibrary(self):
        self.assertRaises(LookupError, rinterface.globalenv.get, "survfit")
        rinterface.globalenv.get("library")(rinterface.StrSexpVector(["survival", ]))
        sfit_R = rinterface.globalenv.get("survfit")
        ok = isinstance(sfit_R, rinterface.SexpClosure)
        self.assertTrue(ok)


    def testGet_functionOnly_lookupError(self):
        # now with the function-only option

        self.assertRaises(LookupError, 
                          rinterface.globalenv.get, "pi", wantfun = True)

    def testGet_functionOnly(self):
        hist = rinterface.globalenv.get("hist", wantfun = False)
        self.assertEqual(rinterface.CLOSXP, hist.typeof)
        rinterface.globalenv["hist"] = rinterface.SexpVector(["foo", ], 
                                                             rinterface.STRSXP)

        hist = rinterface.globalenv.get("hist", wantfun = True)
        self.assertEqual(rinterface.CLOSXP, hist.typeof)
        
    def testGet_emptyString(self):
        self.assertRaises(ValueError, rinterface.globalenv.get, "")

    def testSubscript(self):
        ge = rinterface.globalenv
        obj = rinterface.globalenv.get("letters")
        ge["a"] = obj
        a = rinterface.globalenv["a"]
        ok = ge.get("identical")(obj, a)
        self.assertTrue(ok[0])

    @unittest.skipUnless(is_py3,
                         "unicode keys are only implemented in Python 3.")
    def testSubscript_utf8(self):
        env = rinterface.baseenv['new.env']()
        env[u'呵呵'] = rinterface.IntSexpVector((1,))
        self.assertEqual(1, len(env))
        self.assertEqual(1, len(env[u'呵呵']))
        self.assertEqual(1, env[u'呵呵'][0])

    @unittest.skipUnless(is_py3,
                         "unicode keys are only implemented in Python 3.")
    def testSubscript_missing_utf8(self):
        env = rinterface.baseenv['new.env']()
        self.assertRaises(KeyError, env.__getitem__, u'呵呵')
        
    @unittest.skipIf(is_py3,
                     "Invalid unicode keys is a rpy2 limitation with Python 2.")
    def testSubscript_no_utf8(self):
        env = rinterface.baseenv['new.env']()
        self.assertRaises(ValueError, env.__getitem__, u'x')
        
    def testSubscript_emptyString(self):
        ge = rinterface.globalenv
        self.assertRaises(KeyError, ge.__getitem__, "")

    def testLength(self):
        newEnv = rinterface.globalenv.get("new.env")
        env = newEnv()
        self.assertEqual(0, len(env))
        env["a"] = rinterface.SexpVector([123, ], rinterface.INTSXP)
        self.assertEqual(1, len(env))
        env["b"] = rinterface.SexpVector([123, ], rinterface.INTSXP)
        self.assertEqual(2, len(env))

    def testIter(self):
        newEnv = rinterface.globalenv.get("new.env")
        env = newEnv()
        env["a"] = rinterface.SexpVector([123, ], rinterface.INTSXP)
        env["b"] = rinterface.SexpVector([456, ], rinterface.INTSXP)
        symbols = [x for x in env]
        self.assertEqual(2, len(symbols))
        for s in ["a", "b"]:
            self.assertTrue(s in symbols)

    def testKeys(self):
        newEnv = rinterface.globalenv.get("new.env")
        env = newEnv()
        env["a"] = rinterface.SexpVector([123, ], rinterface.INTSXP)
        env["b"] = rinterface.SexpVector([456, ], rinterface.INTSXP)
        symbols = env.keys()
        self.assertEqual(2, len(symbols))
        for s in ["a", "b"]:
            self.assertTrue(s in symbols)

    def testDel(self):
        env = rinterface.globalenv.get("new.env")()
        env["a"] = rinterface.SexpVector([123, ], rinterface.INTSXP)
        env["b"] = rinterface.SexpVector([456, ], rinterface.INTSXP)
        self.assertEqual(2, len(env))
        del(env['a'])
        self.assertEqual(1, len(env))
        self.assertTrue('b' in env)

    def testDelKeyError(self):
        self.assertRaises(KeyError, rinterface.globalenv.__delitem__, 'foo')

    def testDelBaseError(self):
        self.assertRaises(ValueError, rinterface.baseenv.__delitem__, 'letters')

def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(SexpEnvironmentTestCase)
    return suite

if __name__ == '__main__':
    tr = unittest.TextTestRunner(verbosity = 2)
    tr.run(suite())




import unittest
import sys
import rpy2.robjects as robjects
import rpy2.robjects.conversion as conversion
r = robjects.r

has_numpy = True
try:
    import numpy
    import rpy2.robjects.numpy2ri as rpyn
except:
    has_numpy = False



@unittest.skipUnless(has_numpy, 'numpy is not available in python')
class NumpyConversionsTestCase(unittest.TestCase):

    def setUp(self):
        #self._py2ri = robjects.conversion.py2ri
        #self._ri2py = robjects.conversion.ri2py
        rpyn.activate()

    def tearDown(self):
        rpyn.deactivate()


    def testActivate(self):
        rpyn.deactivate()
        #FIXME: is the following still making sense ?
        self.assertNotEqual(rpyn.py2ri, conversion.py2ri)
        l = len(conversion.py2ri.registry)
        k = set(conversion.py2ri.registry.keys())
        rpyn.activate()
        self.assertTrue(len(conversion.py2ri.registry) > l)
        rpyn.deactivate()
        self.assertEqual(l, len(conversion.py2ri.registry))
        self.assertEqual(k, set(conversion.py2ri.registry.keys()))

    def testActivateTwice(self):
        rpyn.deactivate()
        #FIXME: is the following still making sense ?
        self.assertNotEqual(rpyn.py2ri, conversion.py2ri)
        l = len(conversion.py2ri.registry)
        k = set(conversion.py2ri.registry.keys())
        rpyn.activate()
        rpyn.deactivate()
        rpyn.activate()
        self.assertTrue(len(conversion.py2ri.registry) > l)
        rpyn.deactivate()
        self.assertEqual(l, len(conversion.py2ri.registry))
        self.assertEqual(k, set(conversion.py2ri.registry.keys()))

    def checkHomogeneous(self, obj, mode, storage_mode):
        converted = conversion.py2ri(obj)
        self.assertEqual(r["mode"](converted)[0], mode)
        self.assertEqual(r["storage.mode"](converted)[0], storage_mode)
        self.assertEqual(list(obj), list(converted))
        self.assertTrue(r["is.array"](converted)[0])
        return converted

    def testVectorBoolean(self):
        l = [True, False, True]
        b = numpy.array(l, dtype=numpy.bool_)
        b_r = self.checkHomogeneous(b, "logical", "logical")
        self.assertTupleEqual(tuple(l), tuple(b_r))

    def testVectorInteger(self):
        l = [1, 2, 3]
        i = numpy.array(l, dtype="i")
        i_r = self.checkHomogeneous(i, "numeric", "integer")
        self.assertTupleEqual(tuple(l), tuple(i_r))
        
    def testVectorFloat(self):
        l = [1.0, 2.0, 3.0]
        f = numpy.array(l, dtype="f")
        f_r = self.checkHomogeneous(f, "numeric", "double")
        for orig, conv in zip(l, f_r):
            self.assertTrue(abs(orig-conv) < 0.000001)
        
    def testVectorComplex(self):
        l = [1j, 2j, 3j]
        c = numpy.array(l, dtype=numpy.complex_)
        c_r = self.checkHomogeneous(c, "complex", "complex")
        for orig, conv in zip(l, c_r):
            self.assertTrue(abs(orig.real-conv.real) < 0.000001)
            self.assertTrue(abs(orig.imag-conv.imag) < 0.000001)

    @unittest.skipUnless(sys.version_info[0] < 3,
                         "Test only relevant if Python < 3.")
    def testVectorCharacter(self):
        l = ["a", "b", "c"]
        s = numpy.array(l, dtype="S")
        s_r = self.checkHomogeneous(s, "character", "character")
        self.assertTupleEqual(tuple(l), tuple(s_r))

    def testVectorUnicodeCharacter(self):
        l = [u"a", u"b", u"c"]
        u = numpy.array(l, dtype="U")
        u_r = self.checkHomogeneous(u, "character", "character")
        self.assertTupleEqual(tuple(l), tuple(u_r))

    def testArray(self):

        i2d = numpy.array([[1, 2, 3], [4, 5, 6]], dtype="i")
        i2d_r = conversion.py2ri(i2d)
        self.assertEqual(r["storage.mode"](i2d_r)[0], "integer")
        self.assertEqual(tuple(r["dim"](i2d_r)), (2, 3))

        # Make sure we got the row/column swap right:
        self.assertEqual(r["["](i2d_r, 1, 2)[0], i2d[0, 1])

        f3d = numpy.arange(24, dtype="f").reshape((2, 3, 4))
        f3d_r = conversion.py2ri(f3d)

        self.assertEqual(r["storage.mode"](f3d_r)[0], "double")
        self.assertEqual(tuple(r["dim"](f3d_r)), (2, 3, 4))

        # Make sure we got the row/column swap right:
        self.assertEqual(r["["](f3d_r, 1, 2, 3)[0], f3d[0, 1, 2])

    def testScalar(self):
        i32 = numpy.int32(100)
        i32_r = conversion.py2ri(i32)
        i32_test = numpy.array(i32_r)[0]
        self.assertEqual(i32, i32_test)

        i64 = numpy.int64(100)
        i64_r = conversion.py2ri(i64)
        i64_test = numpy.array(i64_r)[0]
        self.assertEqual(i64, i64_test)

        # 32-bit numpy does not support float128
        if hasattr(numpy, 'float128'):
            f128 = numpy.float128(100.000000003)
            f128_r = conversion.py2ri(f128)
            f128_test = numpy.array(f128_r)[0]
            self.assertEqual(f128, f128_test)

    def testObjectArray(self):
        o = numpy.array([1, "a", 3.2], dtype=numpy.object_)
        o_r = conversion.py2ri(o)
        self.assertEqual(r["mode"](o_r)[0], "list")
        self.assertEqual(r["[["](o_r, 1)[0], 1)
        self.assertEqual(r["[["](o_r, 2)[0], "a")
        self.assertEqual(r["[["](o_r, 3)[0], 3.2)

    def testRecordArray(self):
        rec = numpy.array([(1, 2.3), (2, -0.7), (3, 12.1)],
                          dtype=[("count", "i"), ("value", numpy.double)])
        rec_r = conversion.py2ri(rec)
        self.assertTrue(r["is.data.frame"](rec_r)[0])
        self.assertEqual(tuple(r["names"](rec_r)), ("count", "value"))
        count_r = r["$"](rec_r, "count")
        value_r = r["$"](rec_r, "value")
        self.assertEqual(r["storage.mode"](count_r)[0], "integer")
        self.assertEqual(r["storage.mode"](value_r)[0], "double")
        self.assertEqual(count_r[1], 2)
        self.assertEqual(value_r[2], 12.1)

    def testBadArray(self):
        u = numpy.array([1, 2, 3], dtype=numpy.uint32)
        self.assertRaises(ValueError, conversion.py2ri, u)

    def testAssignNumpyObject(self):
        x = numpy.arange(-10., 10., 1)
        env = robjects.Environment()
        env["x"] = x
        self.assertEqual(1, len(env))
        # do have an R object of the right type ?
        x_r = env["x"]
        self.assertEqual(robjects.rinterface.REALSXP, x_r.typeof)
        #
        self.assertEqual((20,), tuple(x_r.dim))


    def testDataFrameToNumpy(self):
        df = robjects.vectors.DataFrame(dict((('a', 1), ('b', 2))))
        rec = conversion.ri2py(df)
        self.assertEqual(numpy.recarray, type(rec))
        self.assertEqual(1, rec.a[0])
        self.assertEqual(2, rec.b[0])

    def testAtomicVectorToNumpy(self):
        v = robjects.vectors.IntVector((1,2,3))
        a = rpyn.ri2py(v)
        self.assertTrue(isinstance(a, numpy.ndarray))
        self.assertEqual(1, v[0])

    def testRx2(self):
        df = robjects.vectors.DataFrame({
            "A": robjects.vectors.IntVector([1,2,3]),
            "B": robjects.vectors.IntVector([1,2,3])})
        b = df.rx2('B')
        self.assertEquals(tuple((1,2,3)), tuple(b))

def suite():
    if has_numpy:
        return unittest.TestLoader().loadTestsFromTestCase(NumpyConversionsTestCase)
    else:
        return unittest.TestLoader().loadTestsFromTestCase(MissingNumpyDummyTestCase)

if __name__ == '__main__':
    unittest.main()




import unittest
import rpy2.robjects as robjects
import rpy2.robjects.help as rh
rinterface = robjects.rinterface


class PackageTestCase(unittest.TestCase):

    def testInit(self):
        base_help = rh.Package('base')
        self.assertEqual('base', base_help.name)

    def testFetch(self):
        base_help = rh.Package('base')
        f = base_help.fetch('print')
        self.assertTrue('title' in f.sections.keys())

class PageTestCase(unittest.TestCase):
    
    def testInit(self):
        base_help = rh.Package('base')
        p = base_help.fetch('print')
        self.assertEqual('title', p.sections.keys()[0])
    
    def testToDocstring(self):
        base_help = rh.Package('base')
        p = base_help.fetch('print')
        ds = p.to_docstring()
        self.assertEqual('title', ds[:5])
        #self.assertEqual('-----', ds[2])

def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(PackageTestCase)
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(PageTestCase))
    return suite

if __name__ == '__main__':
     unittest.main()




def default_key(x):
    """ Default comparison function """
    return x

def order(seq, key = default_key, reverse = False):
    """ Return the order in which to take the items to obtained
    a sorted sequence."""
    o = list(range(len(seq)))

    def wrap_key(x):
        x = seq[x]
        return key(x)
        
    o.sort(key = wrap_key, reverse = reverse)

    return o





import unittest

# Try to load R ggplot package, and see if it works
from rpy2.rinterface import RRuntimeError
has_ggplot = True
try:
    from rpy2.robjects.lib import ggplot2
except RRuntimeError:
    has_ggplot = False

from rpy2.robjects.packages import importr
datasets = importr('datasets')
mtcars = datasets.__rdata__.fetch('mtcars')['mtcars']

@unittest.skipUnless(has_ggplot, 'ggplot2 package not available in R')
class GGPlot2TestCase(unittest.TestCase):

    def testSetup(self):
        pass

    def tearDown(self):
        pass

    def testGGPlot(self):
        gp = ggplot2.ggplot(mtcars)
        self.assertTrue(isinstance(gp, ggplot2.GGPlot))

    def testAdd(self):
        gp = ggplot2.ggplot(mtcars)
        pp = gp + \
            ggplot2.aes_string(x='wt', y='mpg') + \
            ggplot2.geom_point()
        self.assertTrue(isinstance(pp, ggplot2.GGPlot))

def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(GGPlot2TestCase)
    return suite

if __name__ == '__main__':
     unittest.main()



import os
import warnings
from types import ModuleType
from collections import defaultdict
from warnings import warn
import rpy2.rinterface as rinterface
import rpy2.robjects.lib
from . import conversion
from rpy2.robjects.functions import (SignatureTranslatedFunction, 
                                     docstring_property, 
                                     DocumentedSTFunction)
from rpy2.robjects.constants import NULL
from rpy2.robjects import Environment
from rpy2.robjects.packages_utils import (_libpaths, 
                                          get_packagepath, 
                                          _packages,
                                          default_symbol_r2python,
                                          default_symbol_check_after,
                                          _map_symbols,
                                          _fix_map_symbols)
import rpy2.robjects.help as rhelp

_require = rinterface.baseenv['require']
_library = rinterface.baseenv['library']
_as_env = rinterface.baseenv['as.environment']
_package_has_namespace = rinterface.baseenv['packageHasNamespace']
_system_file = rinterface.baseenv['system.file']
_get_namespace = rinterface.baseenv['getNamespace']
_get_namespace_version = rinterface.baseenv['getNamespaceVersion']
_get_namespace_exports = rinterface.baseenv['getNamespaceExports']
_loaded_namespaces = rinterface.baseenv['loadedNamespaces']
_globalenv = rinterface.globalenv
_new_env = rinterface.baseenv["new.env"]

StrSexpVector = rinterface.StrSexpVector
# Fetching symbols in the namespace "utils" assumes that "utils" is loaded
# (currently the case by default in R).
_data = rinterface.baseenv['::'](StrSexpVector(('utils', )),
                                 StrSexpVector(('data', )))

_reval = rinterface.baseenv['eval']
_options = rinterface.baseenv['options']


def no_warnings(func):
    """ Decorator to run R functions without warning. """
    def run_withoutwarnings(*args, **kwargs):
        warn_i = _options().do_slot('names').index('warn')
        oldwarn = _options()[warn_i][0]
        _options(warn = -1)
        try:
            res = func(*args, **kwargs)
        except Exception as e:
            # restore the old warn setting before propagating
            # the exception up
            _options(warn = oldwarn)
            raise e
        _options(warn = oldwarn)
        return res
    return run_withoutwarnings

@no_warnings
def _eval_quiet(expr):
    return _reval(expr)

# FIXME: should this be part of the API for rinterface ?
#        (may be it is already the case and there is code
#        duplicaton ?)
def reval(string, envir = _globalenv):
    """ Evaluate a string as R code
    :param string: R code
    :type string: a :class:`str`
    :param envir: an environment in which the environment should take place (default: R's global environment)
    """
    p = rinterface.parse(string)
    res = _reval(p, envir = envir)
    return res

def quiet_require(name, lib_loc = None):
    """ Load an R package /quietly/ (suppressing messages to the console). """
    if lib_loc == None:
        lib_loc = "NULL"
    else:
        lib_loc = "\"%s\"" % (lib_loc.replace('"', '\\"'))
    expr_txt = "suppressPackageStartupMessages(base::require(%s, lib.loc=%s))" \
        %(name, lib_loc)
    expr = rinterface.parse(expr_txt)
    ok = _eval_quiet(expr)
    return ok


class PackageData(object):
    """ Datasets in an R package.
    In R datasets can be distributed with a package.

    Datasets can be:

    - serialized R objects

    - R code (that produces the dataset)

    For a given R packages, datasets are stored separately from the rest
    of the code and are evaluated/loaded lazily.

    The lazy aspect has been conserved and the dataset are only loaded
    or generated when called through the method 'fetch()'.
    """
    _packagename = None
    _lib_loc = None
    _datasets = None
    def __init__(self, packagename, lib_loc = rinterface.NULL):
        self._packagename = packagename
        self._lib_loc

    def _init_setlist(self):
        _datasets = dict()
        # 2D array of information about datatsets
        tmp_m = _data(**{'package':StrSexpVector((self._packagename, )),
                         'lib.loc': self._lib_loc})[2]
        nrows, ncols = tmp_m.do_slot('dim')
        c_i = 2
        for r_i in range(nrows):
            _datasets[tmp_m[r_i + c_i * nrows]] = None
            # FIXME: check if instance methods are overriden
        self._datasets = _datasets

    def names(self):
        """ Names of the datasets"""
        if self._datasets is None:
            self._init_setlist()
        return self._datasets.keys()
    
    def fetch(self, name):
        """ Fetch the dataset (loads it or evaluates the R associated
        with it.

        In R, datasets are loaded into the global environment by default
        but this function returns an environment that contains the dataset(s).
        """
        #tmp_env = rinterface.SexpEnvironment()
        if self._datasets is None:
            self._init_setlist()

        if name not in self._datasets:
            raise ValueError('Data set "%s" cannot be found' % name)
        env = _new_env()
        _data(StrSexpVector((name, )),
              **{'package': StrSexpVector((self._packagename, )),
                 'lib.loc': self._lib_loc,
                 'envir': env})
        return Environment(env)

class Package(ModuleType):
    """ Models an R package
    (and can do so from an arbitrary environment - with the caution
    that locked environments should mostly be considered).
     """
    
    _env = None
    __rname__ = None
    _translation = None
    _rpy2r = None
    __fill_rpy2r__ = None
    __update_dict__ = None
    _exported_names = None
    _symbol_r2python = None
    __version__ = None
    __rdata__ = None

    def __init__(self, env, name, translation = {}, 
                 exported_names = None, on_conflict = 'fail',
                 version = None,
                 symbol_r2python = default_symbol_r2python,
                 symbol_check_after = default_symbol_check_after):
        """ Create a Python module-like object from an R environment,
        using the specified translation if defined. 

        - env: R environment
        - name: package name
        - translation: `dict` with R names as keys and corresponding Python
                       names as values
        - exported_names: `set` of names/symbols to expose to instance users
        - on_conflict: 'fail' or 'warn' (default: 'fail')
        - version: version string for the package
        - symbol_r2python: function to convert R symbols into Python symbols.
                           The default translate `.` into `_`.
        - symbol_check_after: function to check the Python symbols obtained
                              from `symbol_r2python`.
        """

        super(Package, self).__init__(name)
        self._env = env
        self.__rname__ = name
        self._translation = translation
        mynames = tuple(self.__dict__)
        self._rpy2r = {}
        if exported_names is None:
            exported_names = set(self._env.keys())
        self._exported_names = exported_names
        self._symbol_r2python = symbol_r2python
        self._symbol_check_after = symbol_check_after
        self.__fill_rpy2r__(on_conflict = on_conflict)
        self._exported_names = self._exported_names.difference(mynames)
        self.__version__ = version
                
    def __update_dict__(self, on_conflict = 'fail'):
        """ Update the __dict__ according to what is in the R environment """
        for elt in self._rpy2r:
            del(self.__dict__[elt])
        self._rpy2r.clear()
        self.__fill_rpy2r__(on_conflict = on_conflict)

    def __fill_rpy2r__(self, on_conflict = 'fail'):
        """ Fill the attribute _rpy2r.

        - on_conflict: 'fail' or 'warn' (default: 'fail')
        """

        assert(on_conflict in ('fail', 'warn'))

        name = self.__rname__

        (symbol_mapping, 
         conflicts, 
         resolutions) = _map_symbols(self._env,
                                     translation = self._translation,
                                     symbol_r2python = self._symbol_r2python,
                                     symbol_check_after = self._symbol_check_after)
        msg_prefix = 'Conflict when converting R symbols'+\
                     ' in the package "%s"' % self.__rname__ +\
                     ' to Python symbols: \n-'
        exception = LibraryError
        _fix_map_symbols(symbol_mapping,
                         conflicts,
                         on_conflict,
                         msg_prefix,
                         exception)
        symbol_mapping.update(resolutions)
        reserved_pynames = set(dir(self))
        for rpyname, rnames in symbol_mapping.items():
            # last paranoid check
            if len(rnames) > 1:
                raise ValueError('Only one R name should be associated with %s (and we have %s)' % (rpyname, str(rnames)))
            rname = rnames[0]
            if rpyname in reserved_pynames:
                raise LibraryError('The symbol ' + rname +\
                                   ' in the package "' + name + '"' +\
                                   ' is conflicting with' +\
                                   ' a Python object attribute')
            self._rpy2r[rpyname] = rname
            if (rpyname != rname) and (rname in self._exported_names):
                self._exported_names.remove(rname)
                self._exported_names.add(rpyname)
            try:
                riobj = self._env[rname]
            except rinterface.RRuntimeError as rre:
                warn(str(rre))
            rpyobj = conversion.ri2ro(riobj)
            if hasattr(rpyobj, '__rname__'):
                rpyobj.__rname__ = rname
            #FIXME: shouldn't the original R name be also in the __dict__ ?
            self.__dict__[rpyname] = rpyobj

    def __repr__(self):
        s = super(Package, self).__repr__()
        return 'rpy2.robjects.packages.Package as a ' + s

# alias
STF = SignatureTranslatedFunction

class SignatureTranslatedPackage(Package):
    """ R package in which the R functions had their signatures 
    'translated' (that this the named parameters were made to 
    to conform Python's rules for vaiable names)."""
    def __fill_rpy2r__(self, on_conflict = 'fail'):
        super(SignatureTranslatedPackage, self).__fill_rpy2r__(on_conflict = on_conflict)
        for name, robj in self.__dict__.items():
            if isinstance(robj, rinterface.Sexp) and robj.typeof == rinterface.CLOSXP:
                self.__dict__[name] = STF(self.__dict__[name],
                                          on_conflict = on_conflict,
                                          symbol_r2python = self._symbol_r2python,
                                          symbol_check_after = self._symbol_check_after)

# alias
STP = SignatureTranslatedPackage

class SignatureTranslatedAnonymousPackage(SignatureTranslatedPackage):
    def __init__(self, string, name):
        env = Environment()
        reval(string, env)
        super(SignatureTranslatedAnonymousPackage, self).__init__(env,
                                                                  name)

# alias
STAP = SignatureTranslatedAnonymousPackage

class InstalledSTPackage(SignatureTranslatedPackage):
    @docstring_property(__doc__)
    def __doc__(self):
        doc = list(['Python representation of an R package.'])
        if not self.__rname__:
            doc.append('<No information available>')
        else:
            try:
                doc.append(rhelp.docstring(self.__rname__,
                                           self.__rname__ + '-package',
                                           sections=['description']))
            except rhelp.HelpNotFoundError as hnf:
                doc.append('[R help was not found]')
        return os.linesep.join(doc)

    def __fill_rpy2r__(self, on_conflict = 'fail'):
        super(SignatureTranslatedPackage, self).__fill_rpy2r__(on_conflict = on_conflict)
        for name, robj in self.__dict__.items():
            if isinstance(robj, rinterface.Sexp) and robj.typeof == rinterface.CLOSXP:
                self.__dict__[name] = DocumentedSTFunction(self.__dict__[name],
                                                           packagename = self.__rname__)


class InstalledPackage(Package):
    @docstring_property(__doc__)
    def __doc__(self):
        doc = list(['Python representation of an R package.',
                    'R arguments:', ''])
        if not self.__rname__:
            doc.append('<No information available>')
        else:
            try:
                doc.append(rhelp.docstring(self.__rname__,
                                           self.__rname__ + '-package',
                                           sections=['description']))
            except rhelp.HelpNotFoundError as hnf:
                doc.append('[R help was not found]')
        return os.linesep.join(doc)


class WeakPackage(Package):
    """
    'Weak' R package, with which looking for symbols results in
    a warning (and a None returned) whenever the desired symbol is
    not found (rather than a traditional `AttributeError`).
    """
    
    def __getattr__(self, name):
        res =self.__dict__.get(name)
        if res is None:
            warnings.warn("The symbol '%s' is not in this R namespace/package." % name)
        return res
    
class LibraryError(ImportError):
    """ Error occuring when importing an R library """
    pass


class InstalledPackages(object):
    """ R packages installed. """
    def __init__(self, lib_loc=None):
        libraryiqr =  _library(**{'lib.loc': lib_loc})
        lib_results_i = libraryiqr.do_slot('names').index('results')
        self.lib_results = libraryiqr[lib_results_i]
        self.nrows, self.ncols = self.lib_results.do_slot('dim')
        self.colnames = self.lib_results.do_slot('dimnames')[1] # column names
        self.lib_packname_i = self.colnames.index('Package')

    def isinstalled(self, packagename):
        if not isinstance(packagename, rinterface.StrSexpVector):
            rname = rinterface.StrSexpVector((packagename, ))
        else:
            if len(packagename) > 1:
                raise ValueError("Only specify one package name at a time.")
            rname = packagename
        nrows, ncols = self.nrows, self.ncols
        lib_results, lib_packname_i = self.lib_results, self.lib_packname_i
        for i in range(0+lib_packname_i*nrows, 
                       nrows*(lib_packname_i+1), 
                       1):
            if lib_results[i] == packagename:
                return True
        return False

    def __iter__(self):
        """ Iterate through rows, yield tuples at each iteration """
        lib_results = self.lib_results
        nrows, ncols = self.nrows, self.ncols
        colrg = range(0, ncols)
        for row_i in range(nrows):
            yield tuple(lib_results[x*nrows+row_i] for x in colrg)

def isinstalled(name,
                lib_loc = None):
    """
    Find whether an R package is installed 
    :param name: name of an R package
    :param lib_loc: specific location for the R library (default: None)

    :rtype: a :class:`bool`
    """
    
    instapack = InstalledPackages(lib_loc)
    return instapack.isinstalled(name)

def importr(name, 
            lib_loc = None,
            robject_translations = {}, 
            signature_translation = True,
            suppress_messages = True,
            on_conflict = 'fail',
            symbol_r2python = default_symbol_r2python,
            symbol_check_after = default_symbol_check_after,
            data = True):
    """ Import an R package.

    Arguments:

    - name: name of the R package

    - lib_loc: specific location for the R library (default: None)

    - robject_translations: dict (default: {})

    - signature_translation: (True or False)

    - suppress_message: Suppress messages R usually writes on the console
      (defaut: True)

    - on_conflict: 'fail' or 'warn' (default: 'fail')

    - symbol_r2python: function to translate R symbols into Python symbols

    - symbol_check_after: function to check the Python symbol obtained
                          from `symbol_r2python`.

    - data: embed a PackageData objects under the attribute 
      name __rdata__ (default: True)

    Return:

    - an instance of class SignatureTranslatedPackage, or of class Package 

    """

    rname = rinterface.StrSexpVector((name, ))

    if suppress_messages:
        ok = quiet_require(name, lib_loc = lib_loc)
    else:
        ok = _require(rinterface.StrSexpVector(rname), 
                      **{'lib.loc': rinterface.StrSexpVector((lib_loc, ))})[0]
    if not ok:
        raise LibraryError("The R package %s could not be imported" %name)
    if _package_has_namespace(rname, 
                              _system_file(package = rname)):
        env = _get_namespace(rname)
        version = _get_namespace_version(rname)[0]
        exported_names = set(_get_namespace_exports(rname))
    else:
        env = _as_env(rinterface.StrSexpVector(['package:'+name, ]))
        exported_names = None
        version = None

    if signature_translation:
        pack = InstalledSTPackage(env, name, 
                                  translation = robject_translations,
                                  exported_names = exported_names,
                                  on_conflict = on_conflict,
                                  version = version,
                                  symbol_r2python = symbol_r2python,
                                  symbol_check_after = symbol_check_after)
    else:
        pack = InstalledPackage(env, name, translation = robject_translations,
                                exported_names = exported_names,
                                on_conflict = on_conflict,
                                version = version,
                                symbol_r2python = symbol_r2python,
                                symbol_check_after = symbol_check_after)
    if data:
        if pack.__rdata__ is not None:
            warn('While importing the R package "%s", the rpy2 Package object is masking a translated R symbol "__rdata__" already present' % name)
        pack.__rdata__ = PackageData(name, lib_loc = lib_loc)

    return pack

def data(package):
    """ Return the PackageData for the given package."""
    return package.__rdata__

def wherefrom(symbol, startenv = rinterface.globalenv):
    """ For a given symbol, return the environment
    this symbol is first found in, starting from 'startenv'.
    """
    env = startenv
    obj = None
    tryagain = True
    while tryagain:
        try:
            obj = env[symbol]
            tryagain = False
        except LookupError as knf:
            env = env.enclos()
            if env.rsame(rinterface.emptyenv):
                tryagain = False
            else:
                tryagain = True
    return conversion.ri2ro(env)




import unittest
import rpy2.rinterface as rinterface

rinterface.initr()

class SexpExtPtrTestCase(unittest.TestCase):

    def setUp(self):
        self.console = rinterface.get_writeconsole_regular()
        def noconsole(x):
            pass
        rinterface.set_writeconsole_regular(noconsole)

    def tearDown(self):
        rinterface.set_writeconsole_regular(self.console)

    def testNewDefault(self):
        pyobject = "ahaha"
        sexp_new = rinterface.SexpExtPtr(pyobject)
        # R External pointer are never copied
        self.assertEqual(rinterface.EXTPTRSXP, sexp_new.typeof)

    def testNewTag(self):
        pyobject = "ahaha"
        sexp_new = rinterface.SexpExtPtr(pyobject, 
                                         tag = rinterface.StrSexpVector("b"))
        self.assertEqual(rinterface.EXTPTRSXP, sexp_new.typeof)
        self.assertEqual('b', sexp_new.__tag__[0])

    def testNewInvalidTag(self):
        pyobject = "ahaha"
        self.assertRaises(TypeError, rinterface.SexpExtPtr,
                          pyobject, tag = True)

    def testNewProtected(self):
        pyobject = "ahaha"
        sexp_new = rinterface.SexpExtPtr(pyobject, 
                                         protected = rinterface.StrSexpVector("c"))
        self.assertEqual(rinterface.EXTPTRSXP, sexp_new.typeof)
        self.assertEqual('c', sexp_new.__protected__[0])

    def testNewInvalidProtected(self):
        pyobject = "ahaha"
        self.assertRaises(TypeError, rinterface.SexpExtPtr,
                          pyobject, protected = True)


def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(SexpExtPtrTestCase)
    return suite

if __name__ == '__main__':
    tr = unittest.TextTestRunner(verbosity = 2)
    tr.run(suite())

